package com.jiuqi.syntax;

import java.math.BigDecimal;
import java.util.GregorianCalendar;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.text.DecimalFormat;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

public final class SyntaxParser
{
    public static final int OP_PLUS = 1;
    public static final int OP_MINUS = 2;
    public static final int OP_SMINUS = 3;
    public static final int OP_PRODUCT = 4;
    public static final int OP_DIV = 5;
    public static final int OP_CPLUS = 6;
    public static final int OP_EQU = 7;
    public static final int OP_WEQU = 8;
    public static final int OP_NEQ = 9;
    public static final int OP_GLT = 10;
    public static final int OP_BLT = 11;
    public static final int OP_GLE = 12;
    public static final int OP_BLE = 13;
    public static final int OP_AND = 14;
    public static final int OP_OR = 15;
    public static final int OP_NOT = 16;
    public static final int OP_ASSIGN = 17;
    public static final int OP_IF = 18;
    public static final int OP_THEN = 19;
    public static final int OP_ELSE = 20;
    public static final String OP_PLUS_NAME = "+";
    public static final String OP_MINUS_NAME = "-";
    public static final String OP_SMINUS_NAME = "-";
    public static final String OP_PRODUCT_NAME = "*";
    public static final String OP_DIV_NAME = "/";
    public static final String OP_CPLUS_NAME = "&";
    public static final String OP_EQU_NAME = "=";
    public static final String OP_WEQU_NAME = "==";
    public static final String OP_NEQ_NAME = "<>";
    public static final String OP_GLT_NAME = ">";
    public static final String OP_BLT_NAME = "<";
    public static final String OP_GLE_NAME = ">=";
    public static final String OP_BLE_NAME = "<=";
    public static final String OP_AND_NAME = "AND";
    public static final String OP_OR_NAME = "OR";
    public static final String OP_NOT_NAME = "NOT";
    public static final String OP_ASSIGN_NAME = "=";
    public static final String OP_IF_NAME = "IF";
    public static final String OP_THEN_NAME = "THEN";
    public static final String OP_ELSE_NAME = "ELSE";
    public static String OP_PLUS_CHINA_NAME;
    public static String OP_MINUS_CHINA_NAME;
    public static String OP_SMINUS_CHINA_NAME;
    public static String OP_PRODUCT_CHINA_NAME;
    public static String OP_DIV_CHINA_NAME;
    public static String OP_CPLUS_CHINA_NAME;
    public static String OP_EQU_CHINA_NAME;
    public static String OP_WEQU_CHINA_NAME;
    public static String OP_NEQ_CHINA_NAME;
    public static String OP_GLT_CHINA_NAME;
    public static String OP_BLT_CHINA_NAME;
    public static String OP_GLE_CHINA_NAME;
    public static String OP_BLE_CHINA_NAME;
    public static String OP_AND_CHINA_NAME;
    public static String OP_OR_CHINA_NAME;
    public static String OP_NOT_CHINA_NAME;
    public static String OP_ASSIGN_CHINA_NAME;
    public static String OP_IF_CHINA_NAME;
    public static String OP_THEN_CHINA_NAME;
    public static String OP_ELSE_CHINA_NAME;
    public static final int SY_OK = 0;
    public static final int SY_UNKNOWN = 1;
    public static final int SY_LACK_LEFT_SMALL_BRACKET = 2;
    public static final int SY_LACK_RIGHT_SMALL_BRACKET = 3;
    public static final int SY_LACK_LEFT_MIDDLE_BRACKET = 4;
    public static final int SY_LACK_RIGHT_MIDDLE_BRACKET = 5;
    public static final int SY_LACK_LEFT_BIG_BRACKET = 6;
    public static final int SY_LACK_RIGHT_BIG_BRACKET = 7;
    public static final int SY_LACK_SIGN = 8;
    public static final int SY_LACK_CHAR = 9;
    public static final int SY_LACK_DIGIT = 10;
    public static final int SY_LACK_LETTER = 11;
    public static final int SY_LACK_QUOTE = 12;
    public static final int SY_LACK_EQUAL = 13;
    public static final int SY_LACK_COMMA = 14;
    public static final int SY_LACK_INT = 15;
    public static final int SY_LACK_REAL = 16;
    public static final int SY_LACK_STRING = 17;
    public static final int SY_LACK_DATE = 18;
    public static final int SY_LACK_BOOL = 19;
    public static final int SY_LACK_AND = 20;
    public static final int SY_LACK_OR = 21;
    public static final int SY_LACK_NOT = 22;
    public static final int SY_LACK_THEN = 23;
    public static final int SY_LACK_ELSE = 24;
    public static final int SY_BRACKET_OVERLOAD = 51;
    public static final int SY_BRACKET_NO_MATCH = 52;
    public static final int SY_QUOTE_BAD_USE = 53;
    public static final int SY_BIG_BRACKET_BAD_USE = 54;
    public static final int SY_INTERVAL_OUTOF_ORDER = 55;
    public static final int SY_INTERVAL_SET_AT_END = 56;
    public static final int SY_NEED_OF_INTERVALSET = 57;
    public static final int SY_EXIST_SPARE_CHAR = 101;
    public static final int SY_INVALID_CHAR = 102;
    public static final int SY_NEED_IF_EXP = 103;
    public static final int SY_NEED_THEN_EXP = 104;
    public static final int SY_NEED_ELSE_EXP = 105;
    public static final int SY_OVELLAP_OPERATOR = 151;
    public static final int SY_TYPE_NO_MATCH = 152;
    public static final int SY_CPLUS_BAD_USE = 153;
    public static final int SY_CPLUS_OUTOF_ORDER = 154;
    public static final int SY_INT_OVER_BORDER = 155;
    public static final int SY_BAD_CELL_EXP = 156;
    public static final int SY_BAD_SIGN = 157;
    public static final int SY_OVERLAP_CONDITION = 158;
    public static final int SY_BAD_CONDITION = 159;
    public static final int SY_OVERLAP_KIND = 160;
    public static final int SY_NO_SUPPORT_STAR = 161;
    public static final int SY_OVERLAP_STAT = 162;
    public static final int SY_OVERLAP_DIMEN = 163;
    public static final int SY_FUNC_NO_EXIST = 201;
    public static final int SY_FUNC_BAD_PARAM = 202;
    public static final int SY_INTERNAL_ERROR = 1001;
    public static final int SY_UNEXPECTED_ERROR = 1002;
    public static final int SY_UNSUPPORT_ERROR = 1003;
    public static final int SY_PARAMS_ERROR = 1004;
    public static final int SY_FORMULA_ERROR = 1005;
    public static final int SY_TASK_NOT_EXIST = 1011;
    public static final int SY_ZB_NOT_EXIST = 1012;
    public static final int SY_TABLE_NOT_EXIST = 1013;
    public static final int SY_CELL_NOT_EXIST = 1014;
    public static final int SY_DICT_NOT_EXIST = 1015;
    public static final int SY_MEAN_NOT_EXIST = 1016;
    public static final int SY_TERM_NEEDED = 1021;
    public static final int SY_TERM_NOT_SUPPORT = 1022;
    public static final int SY_TERM_OVER_BORDER = 1023;
    public static final int SY_ZB_CAN_NOT_READ = 1031;
    public static final int SY_ZB_CAN_NOT_WRITE = 1032;
    public static final int SY_KEY_ZB_READONLY = 1035;
    public static final int SY_MIXED_ZB_READONLY = 1036;
    public static final int SY_OTHRE_TASK_READONLY = 1037;
    public static final int SY_OTHER_TERM_READONLY = 1038;
    public static final int SY_EXPR_ZBMAP_READONLY = 1039;
    public static final int SY_CONDITION_NOT_SUPPORT = 1041;
    public static final int SY_STATISTIC_NOT_SUPPORT = 1042;
    public static final int SY_CONDITION_CONFLICT = 1043;
    public static final int SY_STATISTIC_TYPE_ERROR = 1044;
    public static final int SY_LJ_NOT_SUPPORT = 1045;
    public static final int SY_INVALID_DEST_CELL = 1046;
    public static final int SY_DIMENSION_NOT_EXIST = 1051;
    public static final int SY_DIMENSION_VALUE_ERROR = 1052;
    public static final int SY_DIMENSION_VALUE_MISS = 1053;
    public static final int SY_DIMENSION_STAT_CONFLICT = 1054;
    public static final int SY_DIMENSION_VALUE_RECUR = 1055;
    public static final int SY_TASK_NOT_INSCOPE = 1061;
    public static final int SY_ZB_NOT_INSCOPE = 1062;
    public static final int SY_TABLE_NOT_INSCOPE = 1063;
    public static final int SY_CELL_NOT_INSCOPE = 1064;
    public static final int SY_DICT_NOT_INSCOPE = 1065;
    public static final int SY_TERM_NOT_INSCOPE = 1066;
    public static final int SY_EXTERNAL_EXCEPTION = 9999;
    public static final int SY_CUSTOM_FIRST = 10000;
    public static final double MIN_VALUE = 1.0E-5;
    public static final int MAX_SIGN_LENGTH = 50;
    public static final int SYNTAX_JUDGE = 0;
    public static final int SYNTAX_EVALUATE = 1;
    public static final int SYNTAX_ATTRACT = 2;
    public static final int FLOATCOPYTYPE_ENUM = 0;
    public static final int FLOATCOPYTYPE_SUM = 1;
    public static final int FLOATCOPYTYPE_UPDATE = 2;
    public static final int FLOATCOPYTYPE_MAX_VALUE = 100;
    NiporlanItemFactory niFactory;
    private OptimizeListener optListener;
    public boolean hasStarSyntax;
    public boolean hasCellSyntax;
    public boolean hasMakeNiporlan;
    public boolean hasExpDescription;
    public boolean hasExpVision;
    public boolean hasExpVisionBanlance;
    public boolean isCheckMode;
    public boolean isIgnoreControl;
    private SyntaxListener syntaxListener;
    private static final FunctionListeners globalFunctionListeners;
    private static final CommonFunctionListener commonFunctionListener;
    private FunctionListeners localFunctionListeners;
    public static final String[] COMPARE_OPS;
    public static final String[] COMPARE_CONJ;
    private NiporlanParser niporlanParser;
    private int estimateType;
    private boolean inParseCell;
    private boolean inEffectCheck;
    private int expi;
    private String expStr;
    private String wholeExpStr;
    private char ch;
    private int ret;
    private SyntaxData estimateData;
    private Niporlan niporlan;
    private String expDescription;
    private String defineExpDescription;
    private String expVision;
    private String defineExpVision;
    private int[] hNumArray;
    private int[] lNumArray;
    private double balanceDifferenceValue;
    public static String exceptionMessage;
    private SyntaxData tempSyData;
    private StringBuffer tmpStrBuffer;
    private String tmpStr;
    private String tmpStr2;
    private double tempReal;
    private int tmpInt;
    private boolean tmpBool;
    private double decPos;
    private String ifExpStr;
    private String thenExpStr;
    private String elseExpStr;
    private int thenPos;
    private int elsePos;
    private boolean canAssignValue;
    private boolean ifThenTrue;
    private boolean judgeResult;
    private CommonData attractResult;
    private static final int NI_IF_ITEM = 0;
    private static final int NI_THEN_ITEM = 1;
    private static final int NI_ELSE_ITEM = 2;
    private int niporlanItemType;
    private Stack unUsedSyntaxDatas;
    public static HashMap RESERVED_SIGNS;
    
    static {
        SyntaxParser.OP_PLUS_CHINA_NAME = "加";
        SyntaxParser.OP_MINUS_CHINA_NAME = "减";
        SyntaxParser.OP_SMINUS_CHINA_NAME = "单目减";
        SyntaxParser.OP_PRODUCT_CHINA_NAME = "乘";
        SyntaxParser.OP_DIV_CHINA_NAME = "除";
        SyntaxParser.OP_CPLUS_CHINA_NAME = "连加";
        SyntaxParser.OP_EQU_CHINA_NAME = "等于";
        SyntaxParser.OP_WEQU_CHINA_NAME = "模糊等";
        SyntaxParser.OP_NEQ_CHINA_NAME = "不等于";
        SyntaxParser.OP_GLT_CHINA_NAME = "大于";
        SyntaxParser.OP_BLT_CHINA_NAME = "小于";
        SyntaxParser.OP_GLE_CHINA_NAME = "大于等于";
        SyntaxParser.OP_BLE_CHINA_NAME = "小于等于";
        SyntaxParser.OP_AND_CHINA_NAME = "并且";
        SyntaxParser.OP_OR_CHINA_NAME = "或者";
        SyntaxParser.OP_NOT_CHINA_NAME = "否";
        SyntaxParser.OP_ASSIGN_CHINA_NAME = "等于";
        SyntaxParser.OP_IF_CHINA_NAME = "如果";
        SyntaxParser.OP_THEN_CHINA_NAME = "那么";
        SyntaxParser.OP_ELSE_CHINA_NAME = "否则";
        globalFunctionListeners = new FunctionListeners();
        commonFunctionListener = new CommonFunctionListener();
        SyntaxParser.globalFunctionListeners.add(SyntaxParser.commonFunctionListener);
        COMPARE_OPS = new String[] { "<>", ">=", "<=", "=", ">", "<" };
        COMPARE_CONJ = new String[] { "AND", "OR" };
        SyntaxParser.exceptionMessage = null;
        (SyntaxParser.RESERVED_SIGNS = new HashMap()).put("T", null);
        SyntaxParser.RESERVED_SIGNS.put("TRUE", null);
        SyntaxParser.RESERVED_SIGNS.put("F", null);
        SyntaxParser.RESERVED_SIGNS.put("FALSE", null);
        SyntaxParser.RESERVED_SIGNS.put("NOT", null);
        SyntaxParser.RESERVED_SIGNS.put("ABS", null);
        SyntaxParser.RESERVED_SIGNS.put("INT", null);
        SyntaxParser.RESERVED_SIGNS.put("ROUND", null);
        SyntaxParser.RESERVED_SIGNS.put("MOD", null);
        SyntaxParser.RESERVED_SIGNS.put("POWER", null);
        SyntaxParser.RESERVED_SIGNS.put("TAN", null);
        SyntaxParser.RESERVED_SIGNS.put("ATAN", null);
        SyntaxParser.RESERVED_SIGNS.put("LN", null);
        SyntaxParser.RESERVED_SIGNS.put("LOG10", null);
        SyntaxParser.RESERVED_SIGNS.put("LOG", null);
        SyntaxParser.RESERVED_SIGNS.put("VAL", null);
        SyntaxParser.RESERVED_SIGNS.put("STR", null);
        SyntaxParser.RESERVED_SIGNS.put("L$", null);
        SyntaxParser.RESERVED_SIGNS.put("M$", null);
        SyntaxParser.RESERVED_SIGNS.put("R$", null);
        SyntaxParser.RESERVED_SIGNS.put("POS", null);
        SyntaxParser.RESERVED_SIGNS.put("IDC", null);
        SyntaxParser.RESERVED_SIGNS.put("ISDIGIT", null);
        SyntaxParser.RESERVED_SIGNS.put("LEN", null);
        SyntaxParser.RESERVED_SIGNS.put("ISNULL", null);
        SyntaxParser.RESERVED_SIGNS.put("EXISTCHINESE", null);
        SyntaxParser.RESERVED_SIGNS.put("INLIST", null);
        SyntaxParser.RESERVED_SIGNS.put("DICTVALUE", null);
        SyntaxParser.RESERVED_SIGNS.put("DV", null);
        SyntaxParser.RESERVED_SIGNS.put("GETMEANING", null);
        SyntaxParser.RESERVED_SIGNS.put("STATE", null);
        SyntaxParser.RESERVED_SIGNS.put("FLOATSTAT", null);
        SyntaxParser.RESERVED_SIGNS.put("FLOATCOPY", null);
        SyntaxParser.RESERVED_SIGNS.put("DATEVALUE", null);
        SyntaxParser.RESERVED_SIGNS.put("FORMATDATE", null);
        SyntaxParser.RESERVED_SIGNS.put("DATE", null);
        SyntaxParser.RESERVED_SIGNS.put("YEAR", null);
        SyntaxParser.RESERVED_SIGNS.put("MONTH", null);
        SyntaxParser.RESERVED_SIGNS.put("DAY", null);
        SyntaxParser.RESERVED_SIGNS.put("TODAY", null);
        SyntaxParser.RESERVED_SIGNS.put("NULLBLOB", null);
        SyntaxParser.RESERVED_SIGNS.put("LINKPLUS", null);
        SyntaxParser.RESERVED_SIGNS.put("AUTOSUM", null);
        SyntaxParser.RESERVED_SIGNS.put("SUM", null);
        SyntaxParser.RESERVED_SIGNS.put("MAX", null);
        SyntaxParser.RESERVED_SIGNS.put("MIN", null);
        SyntaxParser.RESERVED_SIGNS.put("AVG", null);
        SyntaxParser.RESERVED_SIGNS.put("COUNT", null);
        SyntaxParser.RESERVED_SIGNS.put("FIRST", null);
        SyntaxParser.RESERVED_SIGNS.put("LAST", null);
    }
    
    public SyntaxParser() {
        this.niFactory = null;
        this.optListener = null;
        this.hasStarSyntax = true;
        this.hasCellSyntax = true;
        this.hasMakeNiporlan = true;
        this.hasExpDescription = true;
        this.hasExpVision = true;
        this.hasExpVisionBanlance = true;
        this.isCheckMode = false;
        this.isIgnoreControl = false;
        this.syntaxListener = null;
        this.localFunctionListeners = new FunctionListeners();
        this.niporlanParser = null;
        this.estimateType = 0;
        this.inParseCell = false;
        this.inEffectCheck = false;
        this.expi = 0;
        this.expStr = "";
        this.wholeExpStr = "";
        this.ch = '\0';
        this.ret = 0;
        this.estimateData = new SyntaxData();
        this.niporlan = null;
        this.expDescription = "";
        this.defineExpDescription = "";
        this.expVision = "";
        this.defineExpVision = "";
        this.hNumArray = new int[2000];
        this.lNumArray = new int[2000];
        this.tempSyData = new SyntaxData();
        this.tmpStrBuffer = new StringBuffer();
        this.tmpStr = "";
        this.tmpStr2 = "";
        this.tempReal = 0.0;
        this.tmpInt = 0;
        this.tmpBool = false;
        this.decPos = 0.0;
        this.ifExpStr = "";
        this.thenExpStr = "";
        this.elseExpStr = "";
        this.attractResult = new CommonData();
        this.niporlanItemType = 1;
        this.unUsedSyntaxDatas = new Stack();
        this.niporlanParser = new NiporlanParser(this);
        this.balanceDifferenceValue = 1.0E-5;
    }
    
    public void setNIFactory(final NiporlanItemFactory niFactory) {
        this.niFactory = niFactory;
    }
    
    public NiporlanItemFactory getNIFactory() {
        return this.niFactory;
    }
    
    public OptimizeListener getOptimizeListener() {
        return this.optListener;
    }
    
    public void setOptimizeListener(final OptimizeListener optListener) {
        this.optListener = optListener;
        if (optListener != null) {
            this.setNIFactory(NiporlanItemFactory.getFactory());
            this.niporlanParser.setUseDataLocator(true);
        }
    }
    
    public void setShareDataCell() {
        this.setNIFactory(NiporlanItemFactory.getFactory());
    }
    
    public SyntaxListener getSyntaxListener() {
        return this.syntaxListener;
    }
    
    public void setSyntaxListener(final SyntaxListener syntaxListener) {
        this.syntaxListener = syntaxListener;
    }
    
    public static final FunctionListeners getGlobalFunctionListeners() {
        return SyntaxParser.globalFunctionListeners;
    }
    
    public FunctionListeners getLocalFunctionListeners() {
        return this.localFunctionListeners;
    }
    
    public FunctionListener getFunctionListener(final String funcName) {
        for (int i = 0; i < this.localFunctionListeners.size(); ++i) {
            if (this.localFunctionListeners.getListener(i).funcExists(funcName)) {
                return this.localFunctionListeners.getListener(i);
            }
        }
        for (int i = 0; i < SyntaxParser.globalFunctionListeners.size(); ++i) {
            if (SyntaxParser.globalFunctionListeners.getListener(i).funcExists(funcName)) {
                return SyntaxParser.globalFunctionListeners.getListener(i);
            }
        }
        return null;
    }
    
    public Function getFunctionByName(final String funcName) {
        for (int i = 0; i < this.localFunctionListeners.size(); ++i) {
            if (this.localFunctionListeners.getListener(i).funcExists(funcName)) {
                return this.localFunctionListeners.getListener(i).funcByName(funcName);
            }
        }
        for (int i = 0; i < SyntaxParser.globalFunctionListeners.size(); ++i) {
            if (SyntaxParser.globalFunctionListeners.getListener(i).funcExists(funcName)) {
                return SyntaxParser.globalFunctionListeners.getListener(i).funcByName(funcName);
            }
        }
        return null;
    }
    
    public String getListenerCodeMessage(final int retCode) {
        for (int i = 0; i < this.localFunctionListeners.size(); ++i) {
            if (!this.localFunctionListeners.getListener(i).getCodeMessage(retCode).equals("")) {
                return this.localFunctionListeners.getListener(i).getCodeMessage(retCode);
            }
        }
        for (int i = 0; i < SyntaxParser.globalFunctionListeners.size(); ++i) {
            if (SyntaxParser.globalFunctionListeners.getListener(i).getCodeMessage(retCode).equals("")) {
                return SyntaxParser.globalFunctionListeners.getListener(i).getCodeMessage(retCode);
            }
        }
        return "";
    }
    
    public String transNiporlanToJavaScript(final Niporlan niporlan, final boolean isCalculateFormula, final TransJavaScriptListener transJavaScriptListener) {
        final StringBuffer jsBuffer = new StringBuffer();
        try {
            if (!niporlan.isEmpty()) {
                jsBuffer.setLength(0);
                boolean isOK;
                if (niporlan.ifExist()) {
                    jsBuffer.append("if (");
                    isOK = this.transNiporlanItemsToJavaScript(niporlan.getIfItems(), false, transJavaScriptListener, jsBuffer);
                    if (isOK) {
                        jsBuffer.append(") ");
                        isOK = this.transNiporlanItemsToJavaScript(niporlan.getThenItems(), isCalculateFormula, transJavaScriptListener, jsBuffer);
                        if (isOK) {
                            jsBuffer.append(";");
                        }
                        if (isOK && niporlan.elseExist()) {
                            jsBuffer.append(" else ");
                            isOK = this.transNiporlanItemsToJavaScript(niporlan.getElseItems(), isCalculateFormula, transJavaScriptListener, jsBuffer);
                            if (isOK) {
                                jsBuffer.append(";");
                            }
                        }
                    }
                }
                else {
                    isOK = this.transNiporlanItemsToJavaScript(niporlan.getThenItems(), isCalculateFormula, transJavaScriptListener, jsBuffer);
                    if (isOK) {
                        jsBuffer.append(";");
                    }
                }
                if (!isOK) {
                    jsBuffer.append("//" + transJavaScriptListener.getErrorMessage());
                }
            }
        }
        catch (final Exception e) {
            e.printStackTrace();
            jsBuffer.append("//转换公式成JavaScript出现错误：").append(e.getMessage());
        }
        return jsBuffer.toString();
    }
    
    private boolean transNiporlanItemsToJavaScript(final ArrayList niporlanItems, final boolean isCalculateFormula, final TransJavaScriptListener transJavaScriptListener, final StringBuffer jsBuffer) {
        final Stack stack = new Stack();
        DataCell SaveSetDataCell = null;
        NiporlanItem item = null;
        String rightItem = null;
        String leftItem = null;
        String param1 = "";
        String param2 = "";
        String param3 = "";
        String param4 = "";
        for (int i = 0; i < niporlanItems.size(); ++i) {
            item = niporlanItems.get(i);
            Label_1470: {
                switch (item.getType()) {
                    case 0: {
                        switch (item.getOp()) {
                            case 1: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "+" + rightItem + ")";
                                break Label_1470;
                            }
                            case 2: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "-" + rightItem + ")";
                                break Label_1470;
                            }
                            case 3: {
                                leftItem = stack.pop();
                                leftItem = "-" + leftItem;
                                break Label_1470;
                            }
                            case 4: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "*" + rightItem + ")";
                                break Label_1470;
                            }
                            case 5: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "/" + rightItem + ")";
                                break Label_1470;
                            }
                            case 7: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "==" + rightItem + ")";
                                break Label_1470;
                            }
                            case 8: {
                                break Label_1470;
                            }
                            case 9: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "!=" + rightItem + ")";
                                break Label_1470;
                            }
                            case 10: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + ">" + rightItem + ")";
                                break Label_1470;
                            }
                            case 11: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "<" + rightItem + ")";
                                break Label_1470;
                            }
                            case 12: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + ">=" + rightItem + ")";
                                break Label_1470;
                            }
                            case 13: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + "<=" + rightItem + ")";
                                break Label_1470;
                            }
                            case 14: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + " && " + rightItem + ")";
                                break Label_1470;
                            }
                            case 15: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                leftItem = "(" + leftItem + " || " + rightItem + ")";
                                break Label_1470;
                            }
                            case 16: {
                                leftItem = stack.pop();
                                leftItem = " !" + rightItem;
                                break Label_1470;
                            }
                            case 17: {
                                rightItem = stack.pop();
                                leftItem = stack.pop();
                                if (rightItem.indexOf("\"") >= 0) {
                                    final StringBuffer strBuffer = new StringBuffer();
                                    for (int j = 0; j < rightItem.length(); ++j) {
                                        if (rightItem.charAt(j) == '\"') {
                                            strBuffer.append('\\').append('\"');
                                        }
                                        else {
                                            strBuffer.append(rightItem.charAt(j));
                                        }
                                    }
                                    rightItem = strBuffer.toString();
                                }
                                leftItem = transJavaScriptListener.getSetCellValueJS(SaveSetDataCell, rightItem);
                                break Label_1470;
                            }
                            default: {
                                return false;
                            }
                        }
                        break;
                    }
                    case 1: {
                        final FunctionListener funcListener = this.getFunctionListener(item.getFuncName());
                        if (funcListener == null) {
                            System.out.println("没有找到函数“" + item.getFuncName() + "”的侦听器！");
                            return false;
                        }
                        final Function func = funcListener.funcByName(item.getFuncName());
                        if (!func.isInfiniteParam) {
                            for (int k = 0; k < func.params.size(); ++k) {
                                final String s = stack.pop();
                                if (func.params.size() - k - 1 == 0) {
                                    param1 = s;
                                }
                                if (func.params.size() - k - 1 == 1) {
                                    param2 = s;
                                }
                                if (func.params.size() - k - 1 == 2) {
                                    param3 = s;
                                }
                                if (func.params.size() - k - 1 == 3) {
                                    param4 = s;
                                }
                            }
                            leftItem = transJavaScriptListener.getFunctionJS(item.getFuncName(), param1, param2, param3, param4);
                            break;
                        }
                        leftItem = stack.pop();
                        final int paramCount = Integer.valueOf(leftItem);
                        final String[] params = new String[paramCount];
                        for (int l = 0; l < paramCount; ++l) {
                            params[paramCount - l - 1] = stack.pop();
                        }
                        leftItem = transJavaScriptListener.getFunctionJS(item.getFuncName(), params);
                        break;
                    }
                    case 2: {
                        leftItem = item.getCommonData().toString();
                        if (item.getCommonData().type == 3) {
                            leftItem = "\"" + leftItem + "\"";
                            break;
                        }
                        break;
                    }
                    case 3: {
                        if (isCalculateFormula && stack.size() == 0) {
                            SaveSetDataCell = item.getDataCell();
                            leftItem = "";
                            break;
                        }
                        leftItem = transJavaScriptListener.getGetCellValueJS(item.getDataCell());
                        break;
                    }
                    default: {
                        return false;
                    }
                }
            }
            stack.push(leftItem);
        }
        final String js = stack.pop();
        jsBuffer.append(js);
        return true;
    }
    
    @Deprecated
    public static final Functions getFunctions() {
        return SyntaxParser.commonFunctionListener.getFunctions();
    }
    
    public int judge(final String expStr, final CommonData resultData) {
        this.estimateType = 0;
        final int syResult = this.estimate(expStr, this.estimateData);
        resultData.assign(this.estimateData);
        return syResult;
    }
    
    public int evaluate(final String expStr) {
        this.estimateType = 1;
        final int syResult = this.estimate(expStr, this.estimateData);
        return syResult;
    }
    
    public int attract(final String expStr, final CommonData resultData) {
        this.estimateType = 2;
        final int syResult = this.estimate(expStr, this.estimateData);
        resultData.assign(this.estimateData);
        return syResult;
    }
    
    public boolean judge(final Niporlan niporlan) {
        return this.niporlanParser.judge(niporlan);
    }
    
    public void evaluate(final Niporlan niporlan) {
        this.niporlanParser.evaluate(niporlan);
    }
    
    public void attract(final Niporlan niporlan, final CommonData resultData) {
        this.niporlanParser.attract(niporlan, resultData);
    }
    
    public static int extendPos(final String subStr, final String mainStr) {
        boolean strStart = false;
        int findPos = -1;
        for (int i = 0; i < mainStr.length() - subStr.length() + 1; ++i) {
            if (mainStr.charAt(i) == '\"') {
                strStart = !strStart;
            }
            if (!strStart) {
                boolean foundOk = true;
                for (int j = 0; j < subStr.length(); ++j) {
                    if (subStr.charAt(j) != mainStr.charAt(i + j)) {
                        foundOk = false;
                        break;
                    }
                }
                if (foundOk) {
                    findPos = i;
                    break;
                }
            }
        }
        return findPos;
    }
    
    public static int extendPos2(final String subStr, final String mainStr) {
        boolean strStart = false;
        int findPos = -1;
        for (int i = 0; i < mainStr.length() - subStr.length() + 1; ++i) {
            if (mainStr.charAt(i) == '\'') {
                strStart = !strStart;
            }
            if (!strStart) {
                boolean foundOk = true;
                for (int j = 0; j < subStr.length(); ++j) {
                    if (subStr.charAt(j) != mainStr.charAt(i + j)) {
                        foundOk = false;
                        break;
                    }
                }
                if (foundOk) {
                    findPos = i;
                    break;
                }
            }
        }
        return findPos;
    }
    
    public static int syntaxPos(String subString, String searchString) {
        if (subString == null) {
            return -1;
        }
        final int subLength = subString.length();
        if (subLength <= 0 || searchString == null) {
            return -1;
        }
        subString = subString.toUpperCase();
        searchString = searchString.toUpperCase();
        boolean strStart = false;
        boolean rawStart = false;
        for (int searchStart = searchString.length() - subLength, i = 0; i <= searchStart; ++i) {
            final char ch = searchString.charAt(i);
            if (!rawStart) {
                if (ch == '\"') {
                    strStart = !strStart;
                    continue;
                }
                if (strStart) {
                    continue;
                }
            }
            if (ch == '\'') {
                rawStart = !rawStart;
            }
            else if (!rawStart) {
                if (ch == subString.charAt(0) && subString.equals(searchString.substring(i, i + subLength))) {
                    return i;
                }
            }
        }
        return -1;
    }
    
    public static int visionSyntaxPos(String subString, String searchString) {
        if (subString == null) {
            return -1;
        }
        final int subLength = subString.length();
        if (subLength <= 0 || searchString == null) {
            return -1;
        }
        subString = subString.toUpperCase();
        searchString = searchString.toUpperCase();
        boolean strStart = false;
        boolean rawStart = false;
        boolean bracketStart = false;
        for (int searchStart = searchString.length() - subLength, i = 0; i <= searchStart; ++i) {
            final char ch = searchString.charAt(i);
            if (!rawStart) {
                if (ch == '\"') {
                    strStart = !strStart;
                    continue;
                }
                if (strStart) {
                    continue;
                }
                if (ch == '[') {
                    bracketStart = true;
                    continue;
                }
                if (ch == ']') {
                    bracketStart = false;
                    continue;
                }
                if (bracketStart) {
                    continue;
                }
            }
            if (ch == '\'') {
                rawStart = !rawStart;
            }
            else if (!rawStart) {
                if (ch == subString.charAt(0) && subString.equals(searchString.substring(i, i + subLength))) {
                    return i;
                }
            }
        }
        return -1;
    }
    
    public static void splitExp(final String expStr, final List list) {
        final int pos1 = syntaxPos(SyntaxParser.COMPARE_CONJ[0], expStr);
        if (pos1 < 1) {
            final int pos2 = syntaxPos(SyntaxParser.COMPARE_CONJ[1], expStr);
            if (pos2 < 1) {
                list.add(expStr.trim());
            }
            else {
                splitExp(expStr.substring(0, pos2), list);
                splitExp(expStr.substring(pos2 + SyntaxParser.COMPARE_CONJ[1].length(), expStr.length()), list);
            }
        }
        else {
            splitExp(expStr.substring(0, pos1), list);
            splitExp(expStr.substring(pos1 + SyntaxParser.COMPARE_CONJ[0].length(), expStr.length()), list);
        }
    }
    
    public static String makeLeftExp(final String expStr) {
        String result = "";
        for (int i = 0; i < SyntaxParser.COMPARE_OPS.length; ++i) {
            final int pos = visionSyntaxPos(SyntaxParser.COMPARE_OPS[i], expStr);
            if (pos > 0) {
                result = expStr.substring(0, pos);
                break;
            }
        }
        return result;
    }
    
    public static String makeRightExp(final String expStr) {
        String result = "";
        for (int i = 0; i < SyntaxParser.COMPARE_OPS.length; ++i) {
            final int pos = visionSyntaxPos(SyntaxParser.COMPARE_OPS[i], expStr);
            if (pos > 0) {
                result = expStr.substring(pos + SyntaxParser.COMPARE_OPS[i].length());
                break;
            }
        }
        return result;
    }
    
    public static String makeBanlanceExp(final String expStr) {
        String result = "";
        for (int i = 0; i < SyntaxParser.COMPARE_OPS.length; ++i) {
            final int pos = visionSyntaxPos(SyntaxParser.COMPARE_OPS[i], expStr);
            if (pos > 0) {
                result = String.valueOf(expStr.substring(0, pos)) + "-(" + expStr.substring(pos + SyntaxParser.COMPARE_OPS[i].length()) + ")";
                break;
            }
        }
        return result;
    }
    
    public static String transExpCellInfo(String sourceExp, final int year, final int termKind, final int term, final int statMode) {
        final StringBuffer destExpBuffer = new StringBuffer();
        final StringBuffer tmpStrBuffer = new StringBuffer();
        final SyntaxData syData = new SyntaxData();
        String remarkStr = "";
        String tmpStr = "";
        int tmpInt = 0;
        final int pos = syntaxPos("//", sourceExp);
        if (pos >= 0) {
            remarkStr = sourceExp.substring(pos, sourceExp.length());
            if (pos > 0) {
                sourceExp = sourceExp.substring(0, pos);
            }
            else {
                sourceExp = "";
            }
        }
        int i = 0;
        while (i < sourceExp.length()) {
            if (sourceExp.charAt(i) == '[') {
                syData.type = 9;
                syData.cdCell.init();
                ++i;
                while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                    ++i;
                }
                if (isFirstSignChar(sourceExp.charAt(i))) {
                    tmpStrBuffer.setLength(0);
                    tmpStrBuffer.append(sourceExp.charAt(i));
                    ++i;
                    while (isSignChar(sourceExp.charAt(i))) {
                        tmpStrBuffer.append(sourceExp.charAt(i));
                        ++i;
                        if (tmpStrBuffer.length() > 50) {}
                    }
                    syData.cdCell.isZB = true;
                    syData.cdCell.zbName = tmpStrBuffer.toString();
                    while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                        ++i;
                    }
                    if (sourceExp.charAt(i) == '$') {
                        syData.cdCell.fetchMean = true;
                        ++i;
                    }
                }
                else {
                    while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                        ++i;
                    }
                    syData.cdCell.isZB = false;
                    tmpStrBuffer.setLength(0);
                    do {
                        tmpStrBuffer.append(sourceExp.charAt(i));
                        ++i;
                        if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                            continue;
                        }
                        break;
                    } while (i < sourceExp.length());
                    try {
                        syData.cdCell.row = Integer.parseInt(tmpStrBuffer.toString());
                    }
                    catch (final Exception ex) {
                        syData.cdCell.row = 0;
                    }
                    while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                        ++i;
                    }
                    if (sourceExp.charAt(i) != ',') {
                        continue;
                    }
                    ++i;
                    tmpStrBuffer.setLength(0);
                    do {
                        tmpStrBuffer.append(sourceExp.charAt(i));
                        ++i;
                        if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                            continue;
                        }
                        break;
                    } while (i < sourceExp.length());
                    try {
                        syData.cdCell.col = Integer.parseInt(tmpStrBuffer.toString());
                    }
                    catch (final Exception ex2) {
                        syData.cdCell.col = 0;
                    }
                }
                while (i < sourceExp.length()) {
                    if (sourceExp.charAt(i) != ' ') {
                        break;
                    }
                    ++i;
                }
                while (i < sourceExp.length()) {
                    if (sourceExp.charAt(i) != ',') {
                        break;
                    }
                    ++i;
                    while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                        ++i;
                    }
                    if (sourceExp.charAt(i) == '\'') {
                        if (syData.cdCell.hasCond) {
                            continue;
                        }
                        syData.cdCell.hasCond = true;
                        ++i;
                        tmpStrBuffer.setLength(0);
                        while (sourceExp.charAt(i) != '\'' && sourceExp.charAt(i) != '\0') {
                            tmpStrBuffer.append(sourceExp.charAt(i));
                            ++i;
                        }
                        if (sourceExp.charAt(i) != '\'') {
                            continue;
                        }
                        syData.cdCell.condition = tmpStrBuffer.toString();
                        ++i;
                        while (i < sourceExp.length()) {
                            if (sourceExp.charAt(i) != ' ') {
                                break;
                            }
                            ++i;
                        }
                    }
                    else {
                        int tempTerm = 0;
                        if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                            tmpStrBuffer.setLength(0);
                            do {
                                tmpStrBuffer.append(sourceExp.charAt(i));
                                ++i;
                                if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                                    continue;
                                }
                                break;
                            } while (i < sourceExp.length());
                            try {
                                tempTerm = Integer.parseInt(tmpStrBuffer.toString());
                            }
                            catch (final Exception ex3) {
                                tempTerm = 0;
                            }
                            while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                                ++i;
                            }
                            tmpStr = "";
                            if (isLetterChar(sourceExp.charAt(i)) || isChinaChar(sourceExp.charAt(i))) {
                                tmpStrBuffer.setLength(0);
                                tmpStrBuffer.append(sourceExp.charAt(i));
                                ++i;
                                while (isLetterChar(sourceExp.charAt(i)) || isChinaChar(sourceExp.charAt(i))) {
                                    tmpStrBuffer.append(sourceExp.charAt(i));
                                    ++i;
                                }
                                tmpStr = tmpStrBuffer.toString();
                            }
                        }
                        else {
                            tmpStrBuffer.setLength(0);
                            while (isLetterChar(sourceExp.charAt(i)) || isChinaChar(sourceExp.charAt(i))) {
                                tmpStrBuffer.append(sourceExp.charAt(i));
                                ++i;
                            }
                            tmpStr = tmpStrBuffer.toString();
                            while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                                ++i;
                            }
                            if (DataCell.statSignToInt(tmpStr) >= 0) {
                                if (syData.cdCell.statMode != 0) {
                                    continue;
                                }
                                syData.cdCell.statMode = DataCell.statSignToInt(tmpStr);
                                continue;
                            }
                            else {
                                tmpInt = DataCell.kindSignToInt(tmpStr);
                                if (tmpInt != 0) {
                                    if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                                        boolean isNext = false;
                                        if (sourceExp.charAt(i) == '+') {
                                            isNext = true;
                                        }
                                        tmpStrBuffer.setLength(0);
                                        do {
                                            tmpStrBuffer.append(sourceExp.charAt(i));
                                            ++i;
                                            if (isDigitChar(sourceExp.charAt(i)) || sourceExp.charAt(i) == '-' || sourceExp.charAt(i) == '+') {
                                                continue;
                                            }
                                            break;
                                        } while (i < sourceExp.length());
                                        try {
                                            tempTerm = Integer.parseInt(tmpStrBuffer.toString());
                                        }
                                        catch (final Exception ex4) {
                                            tempTerm = 0;
                                        }
                                        if (isNext) {
                                            tempTerm += 10000;
                                        }
                                    }
                                    else {
                                        tempTerm = 0;
                                    }
                                }
                                else {
                                    while (isLetterChar(sourceExp.charAt(i)) || isDigitChar(sourceExp.charAt(i))) {
                                        tmpStr = String.valueOf(tmpStr) + sourceExp.charAt(i);
                                        ++i;
                                    }
                                }
                            }
                        }
                        if (tmpStr.equals("")) {
                            if (syData.cdCell.termKind != 0) {
                                continue;
                            }
                            if (syData.cdCell.term != 0) {
                                continue;
                            }
                            syData.cdCell.termKind = 0;
                            syData.cdCell.term = tempTerm;
                        }
                        else {
                            tmpInt = DataCell.kindSignToInt(tmpStr);
                            if (tmpInt == 0) {
                                while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                                    ++i;
                                }
                                if (sourceExp.charAt(i) != '=') {
                                    continue;
                                }
                                ++i;
                                while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                                    ++i;
                                }
                                tmpStrBuffer.setLength(0);
                                if (sourceExp.charAt(i) == '\"') {
                                    ++i;
                                    while (sourceExp.charAt(i) != '\"' && sourceExp.charAt(i) != '\0') {
                                        tmpStrBuffer.append(sourceExp.charAt(i));
                                        ++i;
                                    }
                                    if (sourceExp.charAt(i) != '\"') {
                                        continue;
                                    }
                                    ++i;
                                }
                                else if (sourceExp.charAt(i) == '\'') {
                                    ++i;
                                    while (sourceExp.charAt(i) != '\'' && sourceExp.charAt(i) != '\0') {
                                        tmpStrBuffer.append(sourceExp.charAt(i));
                                        ++i;
                                    }
                                    if (sourceExp.charAt(i) != '\'') {
                                        continue;
                                    }
                                    ++i;
                                }
                                else {
                                    while (sourceExp.charAt(i) != ' ' && sourceExp.charAt(i) != ']' && sourceExp.charAt(i) != ',' && sourceExp.charAt(i) != '\0') {
                                        tmpStrBuffer.append(sourceExp.charAt(i));
                                        ++i;
                                    }
                                }
                                if (syData.cdCell.dimens == null) {
                                    syData.cdCell.dimens = new HashMap(2);
                                }
                                if (syData.cdCell.dimens.get(tmpStr) != null) {
                                    continue;
                                }
                                syData.cdCell.dimens.put(tmpStr, tmpStrBuffer.toString());
                            }
                            else if (tmpInt == 1) {
                                if (tempTerm == 0) {
                                    if (syData.cdCell.termKind != 0) {
                                        continue;
                                    }
                                    syData.cdCell.termKind = 1;
                                    syData.cdCell.term = 0;
                                }
                                else {
                                    if (syData.cdCell.year != 0) {
                                        continue;
                                    }
                                    syData.cdCell.year = tempTerm;
                                }
                            }
                            else {
                                if (syData.cdCell.termKind != 0) {
                                    continue;
                                }
                                if (syData.cdCell.term != 0) {
                                    continue;
                                }
                                syData.cdCell.termKind = tmpInt;
                                syData.cdCell.term = tempTerm;
                            }
                        }
                        while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                            ++i;
                        }
                    }
                }
                while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                    ++i;
                }
                if (sourceExp.charAt(i) != ']') {
                    continue;
                }
                ++i;
                while (i < sourceExp.length() && sourceExp.charAt(i) == ' ') {
                    ++i;
                }
                if (i < sourceExp.length() && sourceExp.charAt(i) == '$') {
                    syData.cdCell.fetchMean = true;
                    ++i;
                }
                syData.cdCell.year = year;
                syData.cdCell.termKind = termKind;
                syData.cdCell.term = term;
                syData.cdCell.statMode = statMode;
                destExpBuffer.append(syData.toString());
            }
            else {
                destExpBuffer.append(sourceExp.charAt(i));
                ++i;
            }
        }
        return String.valueOf(destExpBuffer.toString()) + remarkStr;
    }
    
    public static boolean isDigitChar(final char c) {
        return c >= '0' && c <= '9';
    }
    
    public static boolean isLetterChar(final char c) {
        return (c >= 'A' && c <= 'Z') || (c >= 'a' && c < 'z');
    }
    
    public static boolean isSignChar(final char c) {
        return isLetterChar(c) || isDigitChar(c) || c == '_' || isChinaChar(c);
    }
    
    public static boolean isFirstSignChar(final char c) {
        return isLetterChar(c) || c == '_' || isChinaChar(c);
    }
    
    public static boolean isChinaChar(final char c) {
        return c > 'ÿ';
    }
    
    public static boolean isCompareChar(final char c) {
        return c == '=' || c == '>' || c == '<';
    }
    
    public static boolean isDigit(final String s) {
        if (s.length() > 0) {
            for (int i = 0; i < s.length(); ++i) {
                if (!isDigitChar(s.charAt(i))) {
                    return false;
                }
            }
            return true;
        }
        return true;
    }
    
    public static boolean isSign(final String s) {
        if (s.length() > 0) {
            for (int i = 0; i < s.length(); ++i) {
                if (!isSignChar(s.charAt(i))) {
                    return false;
                }
            }
            return s.charAt(0) < '0' || s.charAt(0) > '9';
        }
        return false;
    }
    
    public static int signPos(String signStr, String mainStr) {
        int findPos = -1;
        signStr = signStr.toUpperCase();
        mainStr = mainStr.toUpperCase();
        if (!isSign(signStr)) {
            return -1;
        }
        boolean strStart = false;
        int i = 0;
        while (i < mainStr.length() - signStr.length() + 1) {
            if (mainStr.charAt(i) == '\"') {
                strStart = !strStart;
                ++i;
            }
            else if (strStart) {
                ++i;
            }
            else if (mainStr.charAt(i) < 'A' || mainStr.charAt(i) > 'Z') {
                ++i;
            }
            else {
                final int starti = i;
                while (i < mainStr.length() && isSignChar(mainStr.charAt(i))) {
                    ++i;
                }
                if (signStr.equals(mainStr.substring(starti, i))) {
                    findPos = starti;
                    break;
                }
                continue;
            }
        }
        return findPos;
    }
    
    public static int asciiLength(final String s) {
        int len = 0;
        for (int i = 0; i < s.length(); ++i) {
            if (isChinaChar(s.charAt(i))) {
                len += 2;
            }
            else {
                ++len;
            }
        }
        return len;
    }
    
    public static int asciiPos(final String subStr, final String mainStr) {
        int pos = mainStr.indexOf(subStr) + 1;
        if (pos > 0) {
            int len = 0;
            for (int i = 0; i < pos - 1; ++i) {
                if (isChinaChar(mainStr.charAt(i))) {
                    len += 2;
                }
                else {
                    ++len;
                }
            }
            pos = len + 1;
        }
        return pos;
    }
    
    public static String asciiCopy(final String s, final int index, int count) {
        final byte[] bytes = s.getBytes();
        if (index <= 0 || index > bytes.length) {
            return "";
        }
        if (index + count - 1 > bytes.length) {
            count = bytes.length - index + 1;
        }
        final byte[] copyBytes = new byte[count];
        for (int i = 0; i < count; ++i) {
            copyBytes[i] = bytes[i + index - 1];
        }
        return new String(copyBytes);
    }
    
    public static boolean existChinese(final String s) {
        for (int i = 0; i < s.length(); ++i) {
            if (isChinaChar(s.charAt(i))) {
                return true;
            }
        }
        return false;
    }
    
    public static long trunc(final double value) {
        return (long)value;
    }
    
    public static double roundFloat(double value, final int decPos) {
        final long decValue = (long)Math.pow(10.0, decPos);
        if (Math.abs(value * decValue - trunc(value * decValue)) - 0.5 > -1.0E-5) {
            if (value < 0.0) {
                value = (double)trunc(value * decValue);
                value = (value - 1.0) / decValue;
            }
            else {
                value = (double)trunc(value * decValue);
                value = (value + 1.0) / decValue;
            }
        }
        else {
            value = (double)trunc(value * decValue);
            value /= decValue;
        }
        return value;
    }
    
    public static double floatMod(final double base, final double divisor) {
        if (divisor == 0.0) {
            return 0.0;
        }
        return base - trunc(base / divisor + 1.0E-5) * divisor;
    }
    
    public static double safeStrToFloat(final String s) {
        try {
            return Double.parseDouble(s);
        }
        catch (final Exception ex) {
            return 0.0;
        }
    }
    
    public static String FloatToStr(final double value, final int decPos, final boolean thousand) {
        if (decPos != 0) {
            final StringBuffer formatStr = new StringBuffer();
            if (thousand) {
                formatStr.append("#,##0.");
            }
            else {
                formatStr.append("0.");
            }
            for (int i = 0; i < decPos; ++i) {
                formatStr.append('0');
            }
            return new DecimalFormat(formatStr.toString()).format(value);
        }
        if (thousand) {
            return new DecimalFormat("#,##0.######").format(value);
        }
        return new DecimalFormat("0.######").format(value);
    }
    
    public static String datetimeToStr(final Calendar date) {
        final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return df.format(date.getTime());
    }
    
    private static int idcGetCharNum(final char c) {
        if (c >= '0' && c <= '9') {
            return c - '0';
        }
        if (c == '#') {
            return 36;
        }
        return Character.toUpperCase(c) - '7';
    }
    
    public static char idc(final String s) {
        if (s.length() != 8) {
            return '\0';
        }
        for (int v = 0; v < s.length(); ++v) {
            if (!isDigitChar(s.charAt(v)) && !isLetterChar(s.charAt(v)) && s.charAt(v) != '#') {
                return '\0';
            }
        }
        int v = 11 - (idcGetCharNum(s.charAt(0)) * 3 + idcGetCharNum(s.charAt(1)) * 7 + idcGetCharNum(s.charAt(2)) * 9 + idcGetCharNum(s.charAt(3)) * 10 + idcGetCharNum(s.charAt(4)) * 5 + idcGetCharNum(s.charAt(5)) * 8 + idcGetCharNum(s.charAt(6)) * 4 + idcGetCharNum(s.charAt(7)) * 2) % 11;
        switch (v) {
            case 10: {
                return 'X';
            }
            case 11: {
                return '0';
            }
            default: {
                return (char)(v + 48);
            }
        }
    }
    
    public static boolean safeStrToDate(final Calendar calendar, final String s, final String format) {
        try {
            if (format == null || format.equals("")) {
                final DateFormat df = DateFormat.getDateInstance();
                df.setLenient(true);
                calendar.getTime().setTime(0L);
                calendar.setTime(df.parse(s));
            }
            else {
                final SimpleDateFormat df2 = new SimpleDateFormat(format);
                df2.setLenient(true);
                df2.toPattern();
                calendar.getTime().setTime(0L);
                calendar.setTime(df2.parse(s));
            }
            return true;
        }
        catch (final Exception ex) {
            calendar.getTime().setTime(0L);
            calendar.set(1, 1899);
            calendar.set(2, 1);
            calendar.set(5, 1);
            return false;
        }
    }
    
    public static String formatDate(final Calendar calendar, String format) {
        try {
            if (format.equals("")) {
                format = "yyyy-MM-dd";
            }
            final SimpleDateFormat df = new SimpleDateFormat(format);
            return df.format(calendar.getTime());
        }
        catch (final Exception ex) {
            return "";
        }
    }
    
    public static void safeEncodeDate(final Calendar calendar, final int year, final int month, final int day) {
        calendar.getTime().setTime(0L);
        calendar.set(year, month - 1, day);
    }
    
    public static int minusDate(final Calendar date1, final Calendar date2) {
        if (date1 == null || date2 == null) {
            return 0;
        }
        int totalDays1 = 0;
        int totalDays2 = 0;
        final int year1 = date1.get(1);
        final int year2 = date2.get(1);
        int i;
        int startYear;
        for (startYear = (i = Math.min(year1, year2)); i <= year1 - 1; ++i) {
            final Calendar calendar = new GregorianCalendar();
            calendar.set(i, 0, 1);
            totalDays1 += calendar.getActualMaximum(6);
        }
        for (i = startYear; i <= year2 - 1; ++i) {
            final Calendar calendar = new GregorianCalendar();
            calendar.set(i, 0, 1);
            totalDays2 += calendar.getMaximum(6);
        }
        totalDays1 += date1.get(6);
        totalDays2 += date2.get(6);
        return totalDays1 - totalDays2;
    }
    
    public int getRetCode() {
        return this.ret;
    }
    
    public String getErrorMessage() {
        return this.getErrorMessage(this.getRetCode());
    }
    
    public String getExpDescription() {
        if (this.defineExpDescription != null && !this.defineExpDescription.equals("")) {
            return this.defineExpDescription;
        }
        return this.expDescription;
    }
    
    public String getDefineExpDescription() {
        return this.defineExpDescription;
    }
    
    public void setDefineExpDescription(final String s) {
        this.defineExpDescription = s;
    }
    
    public String getExpVision() {
        return this.expVision;
    }
    
    public String getDefineExpVision() {
        return this.defineExpVision;
    }
    
    public void setDefineExpVision(final String s) {
        this.defineExpVision = s;
    }
    
    public Niporlan getNiporlan() {
        return this.niporlan;
    }
    
    public String getExpStr() {
        return this.wholeExpStr;
    }
    
    public String getExpStrIfPart() {
        return this.ifExpStr;
    }
    
    public String getExpStrThenPart() {
        if (this.thenExpStr.equals("")) {
            return this.wholeExpStr;
        }
        return this.thenExpStr;
    }
    
    public String getExpStrElsePart() {
        return this.elseExpStr;
    }
    
    public String getErrorMessage(final int retCode) {
        final StringBuffer message = new StringBuffer();
        message.append(this.getCodeMessage(retCode)).append("\n").append(this.expStr).append("\n");
        for (int i = 1; i < this.expi; ++i) {
            message.append(' ');
        }
        message.append('!');
        return message.toString();
    }
    
    public double getBalanceDifferenceValue() {
        return this.balanceDifferenceValue;
    }
    
    public void setBalanceDifferenceValue(final double value) {
        this.balanceDifferenceValue = value + 1.0E-5;
    }
    
    public String getCodeMessage(final int retCode) {
        switch (retCode) {
            case 0: {
                return "公式语法正确";
            }
            case 1: {
                return "未知语法或语法不支持";
            }
            case 2: {
                return "缺少左小括号“(”";
            }
            case 3: {
                return "缺少右小括号“)”";
            }
            case 4: {
                return "缺少左中括号“[”";
            }
            case 5: {
                return "缺少右中括号“]”";
            }
            case 6: {
                return "缺少左大括号“{”";
            }
            case 7: {
                return "缺少右大括号“}”";
            }
            case 8: {
                return "缺少标识符";
            }
            case 9: {
                return "缺少字符";
            }
            case 10: {
                return "缺少数字";
            }
            case 11: {
                return "缺少字母";
            }
            case 12: {
                return "缺少双引号“\"”";
            }
            case 13: {
                return "缺少等于号“=”";
            }
            case 14: {
                return "缺少逗号“,”";
            }
            case 15: {
                return "缺少整数";
            }
            case 16: {
                return "缺少数值";
            }
            case 17: {
                return "缺少字符串";
            }
            case 18: {
                return "缺少日期型";
            }
            case 19: {
                return "缺少布尔值";
            }
            case 20: {
                return "缺少 AND 操作符";
            }
            case 21: {
                return "缺少 OR 操作符";
            }
            case 22: {
                return "缺少 NOT 操作符";
            }
            case 23: {
                return "缺少 THEN 操作符";
            }
            case 24: {
                return "缺少 ELSE 操作符";
            }
            case 51: {
                return "括号太多";
            }
            case 52: {
                return "括号不匹配";
            }
            case 53: {
                return "双引号使用错误";
            }
            case 54: {
                return "大括号使用错误";
            }
            case 55: {
                return "错误的行栏区间设定, 终点小于起点";
            }
            case 56: {
                return "行栏区间设置必须在末尾";
            }
            case 57: {
                return "需要单元区间设置";
            }
            case 101: {
                return "存在多余的字符";
            }
            case 102: {
                return "不合法的字符";
            }
            case 103: {
                return "需要 IF 表达式";
            }
            case 104: {
                return "需要 THEN 表达式";
            }
            case 105: {
                return "需要 ELSE 表达式";
            }
            case 151: {
                return "操作符重叠";
            }
            case 152: {
                return "数据类型不匹配";
            }
            case 153: {
                return "连加符“&”使用错误";
            }
            case 154: {
                return "连加符&终点小于起点";
            }
            case 155: {
                return "整数值越界";
            }
            case 156: {
                return "错误的单元表达式";
            }
            case 157: {
                return "错误的标识符";
            }
            case 158: {
                return "条件设置重复";
            }
            case 159: {
                return "条件设置错误";
            }
            case 160: {
                return "时期设置重复";
            }
            case 161: {
                return "不支持单元格通配符“*”";
            }
            case 162: {
                return "统计模式设置重复";
            }
            case 163: {
                return "维度设置重复";
            }
            case 201: {
                return "函数不存在";
            }
            case 202: {
                return "错误的函数参数";
            }
            case 1001: {
                return "内部错误";
            }
            case 1002: {
                return "未预料错误";
            }
            case 1003: {
                return "功能不被支持";
            }
            case 1004: {
                return "任务参数错误";
            }
            case 1005: {
                return "未知公式错误";
            }
            case 1011: {
                return "任务不存在";
            }
            case 1012: {
                return "指标不存在";
            }
            case 1013: {
                return "报表不存在";
            }
            case 1014: {
                return "单元格不存在";
            }
            case 1015: {
                return "枚举字典不存在";
            }
            case 1016: {
                return "枚举字典含义列不存在";
            }
            case 1021: {
                return "需要明确指定时期";
            }
            case 1022: {
                return "不支持跨时期";
            }
            case 1023: {
                return "时期越界";
            }
            case 1031: {
                return "不能读取指标";
            }
            case 1032: {
                return "不能修改指标";
            }
            case 1035: {
                return "不能修改关键指标";
            }
            case 1036: {
                return "不能修改计算指标、统计指标";
            }
            case 1037: {
                return "不能修改其他任务数据";
            }
            case 1038: {
                return "不能修改其他时期数据";
            }
            case 1039: {
                return "不能修改表达式类型映射关系";
            }
            case 1041: {
                return "不能设置条件";
            }
            case 1042: {
                return "不能设置统计方式";
            }
            case 1043: {
                return "统计条件不在所属关系内";
            }
            case 1044: {
                return "指定统计方式与指标数据类型不匹配";
            }
            case 1045: {
                return "不支持指标累计值";
            }
            case 1046: {
                return "无效的目标单元";
            }
            case 1051: {
                return "维度不存在";
            }
            case 1052: {
                return "维度值设置错误";
            }
            case 1053: {
                return "维度值设置不全面";
            }
            case 1054: {
                return "维度值设置和统计条件冲突";
            }
            case 1055: {
                return "维度值设置重复出现";
            }
            case 1061: {
                return "任务与公式作用域冲突，不可访问";
            }
            case 1062: {
                return "指标与公式作用域冲突，不可访问";
            }
            case 1063: {
                return "报表与公式作用域冲突，不可访问";
            }
            case 1064: {
                return "单元格与公式作用域冲突，不可访问";
            }
            case 1065: {
                return "枚举字典与公式作用域冲突，不可访问";
            }
            case 1066: {
                return "浮动指标、维度不全指标不允许跨时期取数";
            }
            case 9999: {
                return (SyntaxParser.exceptionMessage == null) ? "外部异常错误" : SyntaxParser.exceptionMessage;
            }
            default: {
                final String codeMessage = this.getListenerCodeMessage(retCode);
                if (!codeMessage.equals("")) {
                    return codeMessage;
                }
                return "未定义的错误";
            }
        }
    }
    
    private int estimate(String estimateStr, final SyntaxData syData) {
        final StringBuffer strBuffer = new StringBuffer();
        this.expi = 0;
        final int pos = syntaxPos("//", estimateStr);
        if (pos >= 0) {
            if (pos > 0) {
                estimateStr = estimateStr.substring(0, pos);
            }
            else {
                estimateStr = "";
            }
        }
        boolean strStart = false;
        boolean bracketStart = false;
        int bracketNum = 0;
        estimateStr = estimateStr.trim();
        this.expStr = estimateStr;
        this.wholeExpStr = this.expStr;
        for (int i = 0; i < estimateStr.length(); ++i) {
            this.ch = estimateStr.charAt(i);
            if (!strStart && this.ch == '(') {
                ++bracketNum;
            }
            if (bracketNum > 10) {
                this.ret = 51;
                syData.type = 0;
                return this.ret;
            }
            if (!strStart && this.ch == ')') {
                --bracketNum;
            }
            if (bracketNum < 0) {
                this.ret = 52;
                syData.type = 0;
                return this.ret;
            }
            if (this.ch == '\"') {
                strStart = !strStart;
            }
            if (!strStart) {
                if (this.ch == '[') {
                    bracketStart = true;
                }
                else if (this.ch == ']' || this.ch == '\'') {
                    bracketStart = false;
                }
            }
            if (!bracketStart || this.ch != ' ') {
                if (!strStart) {
                    strBuffer.append(Character.toUpperCase(this.ch));
                }
                else {
                    strBuffer.append(this.ch);
                }
            }
        }
        if (bracketNum != 0) {
            this.ret = 52;
            syData.type = 0;
            return this.ret;
        }
        if (strStart) {
            this.ret = 53;
            syData.type = 0;
            return this.ret;
        }
        this.expStr = strBuffer.toString();
        this.wholeExpStr = this.expStr;
        if (this.expStr.length() == 0) {
            if (this.hasMakeNiporlan) {
                this.niporlan = new Niporlan();
            }
            this.ret = 0;
            if (this.estimateType == 0) {
                syData.type = 5;
                syData.cdBool = true;
                syData.isNull = false;
            }
            else {
                syData.type = 0;
            }
            return this.ret;
        }
        if (this.hasStarSyntax) {
            int hNumNum = 0;
            int lNumNum = 0;
            this.inEffectCheck = false;
            this.expi = 0;
            this.getChar();
            if (this.ch != '{') {
                do {
                    this.getChar();
                } while (this.ch != '{' && this.ch != '\0');
            }
            if (this.ch == '{') {
                this.getChar();
                this.skipSpaces();
                if (this.ch == '}') {
                    hNumNum = 0;
                }
                else {
                    do {
                        int saveExpI = this.expi;
                        this.wholeExp(syData);
                        if (this.ret != 0 || syData.type != 1) {
                            this.expi = saveExpI;
                            return this.ret = 15;
                        }
                        final int startI = syData.cdInt;
                        if (this.ch == '~') {
                            this.getChar();
                            saveExpI = this.expi;
                            this.wholeExp(syData);
                            if (this.ret != 0 || syData.type != 1) {
                                this.expi = saveExpI;
                                return this.ret = 15;
                            }
                            final int endI = syData.cdInt;
                            if (endI < startI) {
                                this.expi = saveExpI;
                                return this.ret = 55;
                            }
                            for (int j = startI; j <= endI; ++j) {
                                ++hNumNum;
                                this.hNumArray[hNumNum] = j;
                            }
                        }
                        else {
                            ++hNumNum;
                            this.hNumArray[hNumNum] = startI;
                        }
                        if (this.ch != ',' && this.ch != '\0' && this.ch != '}') {
                            return this.ret = 102;
                        }
                        if (this.ch == '\0') {
                            this.ret = 7;
                            this.expi = saveExpI;
                            return this.ret;
                        }
                        if (this.ch != ',') {
                            continue;
                        }
                        this.getChar();
                    } while (this.ch != '}' && this.ch != '\0');
                }
                int saveExpI = this.expi;
                this.getChar();
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch != '{' && this.ch != '\0') {
                    this.expi = saveExpI;
                    return this.ret = 56;
                }
            }
            if (this.ch == '{') {
                this.getChar();
                this.skipSpaces();
                if (this.ch == '}') {
                    lNumNum = 0;
                }
                else {
                    do {
                        int saveExpI = this.expi;
                        this.wholeExp(syData);
                        if (this.ret != 0 || syData.type != 1) {
                            this.expi = saveExpI;
                            return this.ret = 15;
                        }
                        final int startI = syData.cdInt;
                        if (this.ch == '~') {
                            this.getChar();
                            saveExpI = this.expi;
                            this.wholeExp(syData);
                            if (this.ret != 0 || syData.type != 1) {
                                this.expi = saveExpI;
                                return this.ret = 15;
                            }
                            final int endI = syData.cdInt;
                            if (endI < startI) {
                                this.expi = saveExpI;
                                return this.ret = 55;
                            }
                            for (int j = startI; j <= endI; ++j) {
                                ++lNumNum;
                                this.lNumArray[lNumNum] = j;
                            }
                        }
                        else {
                            ++lNumNum;
                            this.lNumArray[lNumNum] = startI;
                        }
                        if (this.ch != ',' && this.ch != '\0' && this.ch != '}') {
                            return this.ret = 102;
                        }
                        if (this.ch == '\0') {
                            this.ret = 7;
                            this.expi = saveExpI;
                            return this.ret;
                        }
                        if (this.ch != ',') {
                            continue;
                        }
                        this.getChar();
                    } while (this.ch != '}' && this.ch != '\0');
                }
                int saveExpI = this.expi;
                this.getChar();
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch != '\0') {
                    this.expi = saveExpI;
                    return this.ret = 56;
                }
            }
            this.expi = 0;
            int p = extendPos("{", this.expStr);
            if (p >= 0) {
                this.expStr = this.expStr.substring(0, p);
            }
            final ArrayList lanAreas = new ArrayList();
            final ArrayList hangAreas = new ArrayList();
            this.searchWildcardArea(this.expStr, hangAreas, lanAreas, true);
            p = 0;
            if (extendPos("[*", this.expStr) >= 0) {
                ++p;
            }
            if (extendPos(",*", this.expStr) >= 0) {
                p += 2;
            }
            switch (p) {
                case 0: {
                    hNumNum = 1;
                    lNumNum = 1;
                    break;
                }
                case 1: {
                    if (hNumNum == 0 && hangAreas.size() > 0) {
                        hNumNum = hangAreas.size();
                        for (int k = 1; k <= hNumNum; ++k) {
                            this.hNumArray[k] = hangAreas.get(k - 1);
                        }
                    }
                    if (hNumNum == 0) {
                        return this.ret = 57;
                    }
                    lNumNum = 1;
                    break;
                }
                case 2: {
                    if (hNumNum == 0 && lanAreas.size() > 0) {
                        hNumNum = lanAreas.size();
                        for (int k = 1; k <= hNumNum; ++k) {
                            this.hNumArray[k] = lanAreas.get(k - 1);
                        }
                    }
                    if (hNumNum == 0) {
                        return this.ret = 57;
                    }
                    lNumNum = hNumNum;
                    hNumNum = 1;
                    for (int k = 1; k <= lNumNum; ++k) {
                        this.lNumArray[k] = this.hNumArray[k];
                    }
                    break;
                }
                case 3: {
                    if (hNumNum == 0 && hangAreas.size() > 0) {
                        hNumNum = hangAreas.size();
                        for (int k = 1; k <= hNumNum; ++k) {
                            this.hNumArray[k] = hangAreas.get(k - 1);
                        }
                    }
                    if (lNumNum == 0 && lanAreas.size() > 0) {
                        lNumNum = lanAreas.size();
                        for (int k = 1; k <= lNumNum; ++k) {
                            this.lNumArray[k] = lanAreas.get(k - 1);
                        }
                    }
                    if (hNumNum == 0 || lNumNum == 0) {
                        return this.ret = 57;
                    }
                    break;
                }
            }
            final String rawExpStr = this.expStr;
            for (int r = 1; r <= hNumNum; ++r) {
                strBuffer.setLength(0);
                strBuffer.append(rawExpStr);
                do {
                    p = extendPos("[*", strBuffer.toString());
                    if (p >= 0) {
                        strBuffer.deleteCharAt(p + 1);
                        strBuffer.insert(p + 1, this.hNumArray[r]);
                    }
                } while (p >= 0);
                this.expStr = strBuffer.toString();
                final String midExpStr = this.expStr;
                for (int c = 1; c <= lNumNum; ++c) {
                    strBuffer.setLength(0);
                    strBuffer.append(midExpStr);
                    do {
                        p = extendPos(",*", strBuffer.toString());
                        if (p >= 0) {
                            strBuffer.deleteCharAt(p + 1);
                            strBuffer.insert(p + 1, this.lNumArray[c]);
                        }
                    } while (p >= 0);
                    this.expStr = strBuffer.toString();
                    if (this.defineExpVision != null && !this.defineExpVision.equals("")) {
                        strBuffer.setLength(0);
                        strBuffer.append(this.defineExpVision);
                        do {
                            p = extendPos("[*", strBuffer.toString());
                            if (p >= 0) {
                                strBuffer.deleteCharAt(p + 1);
                                strBuffer.insert(p + 1, this.hNumArray[r]);
                            }
                        } while (p >= 0);
                        do {
                            p = extendPos(",*", strBuffer.toString());
                            if (p >= 0) {
                                strBuffer.deleteCharAt(p + 1);
                                strBuffer.insert(p + 1, this.lNumArray[c]);
                            }
                        } while (p >= 0);
                        this.expVision = strBuffer.toString();
                    }
                    else {
                        this.expVision = "";
                    }
                    this.expCheck(syData);
                    if (this.ret == 1014) {
                        this.finishOneExpress(true);
                    }
                    else {
                        if (this.ret != 0) {
                            return this.ret;
                        }
                        this.finishOneExpress(false);
                    }
                }
            }
            return this.ret = 0;
        }
        if (syntaxPos("{", this.expStr) >= 0) {
            this.expi = syntaxPos("{", this.expStr);
            return this.ret = 54;
        }
        this.expVision = this.defineExpVision;
        this.expCheck(syData);
        if (this.ret == 0) {
            this.finishOneExpress(false);
        }
        return this.ret;
    }
    
    private String searchTableName(final String s, int beginPos) {
        String result = "";
        while (beginPos >= 0) {
            if (s.charAt(beginPos) != ' ') {
                break;
            }
            --beginPos;
        }
        while (beginPos >= 0 && isSignChar(s.charAt(beginPos))) {
            result = String.valueOf(s.charAt(beginPos)) + result;
            --beginPos;
        }
        if (result.equals("IF") || result.equals("THEN") || result.equals("ELSE") || result.equals("AND") || result.equals("OR") || result.equals("NOT")) {
            result = "";
        }
        return result;
    }
    
    private void searchWildcardArea(String expStr, final ArrayList hangAreas, final ArrayList lanAreas, final boolean attractWildcardArea) {
        String newExpStr = "";
        final ArrayList wildcardInfos = new ArrayList();
        hangAreas.clear();
        lanAreas.clear();
        for (int i = syntaxPos("[", expStr); i >= 0; i = syntaxPos("[", expStr)) {
            boolean found = false;
            boolean strStart = false;
            int j;
            for (j = i; j < expStr.length(); ++j) {
                if (expStr.charAt(j) == '\"') {
                    strStart = !strStart;
                }
                if (!strStart && expStr.charAt(j) == ']') {
                    found = true;
                    break;
                }
            }
            if (!found) {
                break;
            }
            String taskTag = "";
            int taskTagPos;
            for (taskTagPos = j + 1; taskTagPos < expStr.length() && expStr.charAt(taskTagPos) == ' '; ++taskTagPos) {}
            if (taskTagPos < expStr.length() && expStr.charAt(taskTagPos) == '@') {
                String cellStr = "";
                ++taskTagPos;
                while (taskTagPos < expStr.length() && isDigitChar(expStr.charAt(taskTagPos))) {
                    cellStr = String.valueOf(cellStr) + expStr.charAt(taskTagPos);
                    ++taskTagPos;
                }
                taskTag = cellStr;
            }
            String cellStr = expStr.substring(i + 1, j);
            int dot = syntaxPos(",", cellStr);
            String cellStr2;
            String cellStr3;
            if (dot >= 0) {
                cellStr2 = cellStr.substring(0, dot);
                cellStr3 = cellStr.substring(dot + 1);
                dot = syntaxPos(",", cellStr3);
                String cellStr4;
                if (dot >= 0) {
                    cellStr4 = cellStr3.substring(dot + 1);
                    cellStr3 = cellStr3.substring(0, dot);
                }
                else {
                    cellStr4 = "";
                }
                if (!cellStr3.equals("") && cellStr3.charAt(0) == '\'') {
                    if (cellStr4.equals("")) {
                        cellStr4 = cellStr3;
                    }
                    else {
                        cellStr4 = String.valueOf(cellStr3) + "," + cellStr4;
                    }
                    cellStr3 = "";
                }
                if (cellStr3.equals("")) {
                    if (cellStr2.equals("*")) {
                        cellStr3 = "*";
                        cellStr = String.valueOf(cellStr2) + "," + cellStr3;
                    }
                    else {
                        int error;
                        int cellIndex;
                        if (cellStr2.equals("") || cellStr2.charAt(0) < '0' || cellStr2.charAt(0) > '9') {
                            error = 0;
                            cellIndex = 0;
                        }
                        else {
                            error = -1;
                            cellIndex = 0;
                            try {
                                cellIndex = Integer.parseInt(cellStr2);
                            }
                            catch (final Exception ex) {
                                error = 0;
                            }
                        }
                        if (error < 0 && cellIndex > 0) {
                            cellStr3 = "*";
                            cellStr = String.valueOf(cellStr2) + "," + cellStr3;
                        }
                        else {
                            cellStr3 = "";
                            cellStr = cellStr2;
                        }
                    }
                    if (!cellStr4.equals("")) {
                        cellStr = String.valueOf(cellStr) + "," + cellStr4;
                    }
                }
            }
            else {
                cellStr2 = cellStr;
                if (cellStr2.equals("*")) {
                    cellStr3 = "*";
                    cellStr = String.valueOf(cellStr2) + "," + cellStr3;
                }
                else {
                    int error;
                    int cellIndex;
                    if (cellStr2.equals("") || cellStr2.charAt(0) < '0' || cellStr2.charAt(0) > '9') {
                        error = 0;
                        cellIndex = 0;
                    }
                    else {
                        error = -1;
                        cellIndex = 0;
                        try {
                            cellIndex = Integer.parseInt(cellStr2);
                        }
                        catch (final Exception ex2) {
                            error = 0;
                        }
                    }
                    if (error < 0 && cellIndex > 0) {
                        cellStr3 = "*";
                        cellStr = String.valueOf(cellStr2) + "," + cellStr3;
                    }
                    else {
                        cellStr3 = "";
                    }
                }
            }
            final String tableName = this.searchTableName(expStr, i - 1);
            if (cellStr2.equals("*") && cellStr3.equals("*")) {
                WildcardInfo info = new WildcardInfo();
                wildcardInfos.add(info);
                info.taskName = taskTag;
                info.tableName = tableName;
                info.isLan = true;
                info.number = 0;
                info = new WildcardInfo();
                wildcardInfos.add(info);
                info.taskName = taskTag;
                info.tableName = tableName;
                info.isLan = false;
                info.number = 0;
            }
            else if (cellStr2.equals("*")) {
                int error;
                int cellIndex;
                if (cellStr3.equals("") || cellStr3.charAt(0) < '0' || cellStr3.charAt(0) > '9') {
                    error = 0;
                    cellIndex = 0;
                }
                else {
                    error = -1;
                    cellIndex = 0;
                    try {
                        cellIndex = Integer.parseInt(cellStr3);
                    }
                    catch (final Exception ex3) {
                        error = 0;
                    }
                }
                if (error < 0 && cellIndex > 0) {
                    final WildcardInfo info = new WildcardInfo();
                    wildcardInfos.add(info);
                    info.taskName = taskTag;
                    info.tableName = tableName;
                    info.isLan = false;
                    info.number = cellIndex;
                }
            }
            else if (cellStr3.equals("*")) {
                int error;
                int cellIndex;
                if (cellStr2.equals("") || cellStr2.charAt(0) < '0' || cellStr2.charAt(0) > '9') {
                    error = 0;
                    cellIndex = 0;
                }
                else {
                    error = -1;
                    cellIndex = 0;
                    try {
                        cellIndex = Integer.parseInt(cellStr2);
                    }
                    catch (final Exception ex4) {
                        error = 0;
                    }
                }
                if (error < 0 && cellIndex > 0) {
                    final WildcardInfo info = new WildcardInfo();
                    wildcardInfos.add(info);
                    info.taskName = taskTag;
                    info.tableName = tableName;
                    info.isLan = true;
                    info.number = cellIndex;
                }
            }
            newExpStr = String.valueOf(newExpStr) + expStr.substring(0, i) + "[" + cellStr + "]";
            expStr = expStr.substring(j + 1);
        }
        newExpStr = (expStr = String.valueOf(newExpStr) + expStr);
        if (attractWildcardArea && wildcardInfos.size() > 0) {
            this.getWildcardAreas(wildcardInfos, hangAreas, lanAreas);
        }
    }
    
    private int getCellData(final DataCell dataCell, final CommonData value) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getValue(new SyntaxEvent(this, dataCell, value));
        }
        return 1;
    }
    
    private int setCellData(final DataCell dataCell, final CommonData value) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.setValue(new SyntaxEvent(this, dataCell, value));
        }
        return 1;
    }
    
    private int getStatData(final DataCell dataCell, final CommonData value) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getStatValue(new SyntaxEvent(this, dataCell, value));
        }
        return 1;
    }
    
    private int getWildcardAreas(final ArrayList wildcardInfos, final ArrayList hangAreas, final ArrayList lanAreas) {
        if (this.syntaxListener != null) {
            final WildcardInfo[] infos = new WildcardInfo[wildcardInfos.size()];
            for (int i = 0; i < wildcardInfos.size(); ++i) {
                infos[i] = wildcardInfos.get(i);
            }
            final WildcardAreasEvent event = new WildcardAreasEvent(this, infos);
            this.syntaxListener.getWildcardAreas(event);
            hangAreas.clear();
            if (event.hangAreas != null) {
                for (int j = 0; j < event.hangAreas.length; ++j) {
                    hangAreas.add(new Integer(event.hangAreas[j]));
                }
            }
            lanAreas.clear();
            if (event.lanAreas != null) {
                for (int j = 0; j < event.lanAreas.length; ++j) {
                    lanAreas.add(new Integer(event.lanAreas[j]));
                }
            }
            return 0;
        }
        return 1;
    }
    
    private int getInDict(final DictMeaningEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getDictMeaning(event);
        }
        return 1;
    }
    
    private int getDictValue(final DictMeaningEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getDictMeaning(event);
        }
        return 1;
    }
    
    private int getUnitState(final UnitStateEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getUnitState(event);
        }
        return 1;
    }
    
    private int getLinkPlus(final LinkPlusEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getLinkPlus(event);
        }
        return 1;
    }
    
    private int getFloatStat(final FloatStatEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.getFloatStat(event);
        }
        return 1;
    }
    
    private int execFloatCopy(final FloatCopyEvent event) {
        if (this.syntaxListener != null) {
            return this.syntaxListener.execFloatCopy(event);
        }
        return 1;
    }
    
    private void finishOneExpress(final boolean hasError) {
        if (this.syntaxListener != null) {
            this.syntaxListener.finishOneExpress(hasError);
        }
    }
    
    private void getChar() {
        if (this.expi >= 0 && this.expi < this.expStr.length()) {
            this.ch = this.expStr.charAt(this.expi);
            ++this.expi;
        }
        else {
            this.ch = '\0';
        }
    }
    
    private char getNextChar() {
        if (this.expi >= 0 && this.expi < this.expStr.length()) {
            return this.expStr.charAt(this.expi);
        }
        return '\0';
    }
    
    private void skipSpaces() {
        while (this.ch == ' ') {
            this.getChar();
        }
    }
    
    private int getInteger() {
        final StringBuffer strBuffer = new StringBuffer();
        if (this.ch == ' ') {
            this.skipSpaces();
        }
        if (this.ch == '+' || this.ch == '-') {
            if (this.ch == '-') {
                strBuffer.append(this.ch);
            }
            this.getChar();
        }
        if (this.ch == ' ') {
            this.skipSpaces();
        }
        if (this.ch < '0' || this.ch > '9') {
            this.ret = 102;
            return 0;
        }
        do {
            strBuffer.append(this.ch);
            this.getChar();
            if (this.ch >= '0') {
                continue;
            }
            break;
        } while (this.ch <= '9');
        try {
            return Integer.parseInt(strBuffer.toString());
        }
        catch (final Exception ex) {
            this.ret = 15;
            return 0;
        }
    }
    
    private void addCellCaption(final DataCell dataCell) {
        final SyntaxCellTitleEvent event = new SyntaxCellTitleEvent(this, dataCell, "");
        if (this.syntaxListener != null) {
            this.syntaxListener.getCellTitle(event);
        }
        if (event.cellTitle == null || event.cellTitle.equals("")) {
            this.addStringCaption(dataCell.toString());
        }
        else {
            this.addStringCaption(event.cellTitle);
        }
    }
    
    private void addStringCaption(final String s) {
        if (!this.inParseCell && this.inEffectCheck) {
            this.expDescription = String.valueOf(this.expDescription) + s;
        }
    }
    
    private void addStringCaption(final char c) {
        this.addStringCaption(String.valueOf(c));
    }
    
    private void addCellVision(final DataCell dataCell) {
        this.addStringVision(dataCell.toString());
    }
    
    private void addStringVision(final String s) {
        if ((this.defineExpVision == null || this.defineExpVision.equals("")) && this.expVision.indexOf(String.valueOf(s) + ";") < 0) {
            this.expVision = String.valueOf(this.expVision) + s + ";";
        }
    }
    
    private void writeNiporlan(final NiporlanItem ni) {
        if (this.niporlan != null) {
            switch (this.niporlanItemType) {
                case 0: {
                    this.niporlan.addIfItem(ni);
                    break;
                }
                case 1: {
                    this.niporlan.addThenItem(ni);
                    break;
                }
                case 2: {
                    this.niporlan.addElseitem(ni);
                    break;
                }
            }
        }
    }
    
    private void writeNiporlan(final int op) {
        if (this.niporlan == null) {
            return;
        }
        if (this.niFactory != null) {
            this.writeNiporlan(this.niFactory.getNiporlanItem(op));
        }
        else {
            this.writeNiporlan(new NiporlanItem(op));
        }
    }
    
    private void writeNiporlan(final String funcName) {
        if (this.niporlan == null) {
            return;
        }
        if (this.niFactory != null) {
            this.writeNiporlan(this.niFactory.getNiporlanItem(funcName));
        }
        else {
            this.writeNiporlan(new NiporlanItem(funcName));
        }
    }
    
    private void writeNiporlan(final CommonData commonData) {
        if (this.niporlan == null) {
            return;
        }
        if (this.niFactory != null) {
            this.writeNiporlan(this.niFactory.getNiporlanItem(commonData));
        }
        else {
            this.writeNiporlan(new NiporlanItem(commonData));
        }
    }
    
    private void writeNiporlan(final DataCell dataCell) {
        if (this.niporlan == null) {
            return;
        }
        NiporlanItem ni = null;
        if (this.niFactory != null) {
            ni = this.niFactory.getNiporlanItem(dataCell);
        }
        else {
            ni = new NiporlanItem(dataCell);
        }
        this.writeNiporlan(ni);
        if (this.optListener != null) {
            this.optListener.finishOneNiporlanItem(ni);
        }
    }
    
    private void writeNiporlan(final Object object) {
        if (this.niporlan == null) {
            return;
        }
        if (this.niFactory != null) {
            this.writeNiporlan(this.niFactory.getNiporlanItem(object));
        }
        else {
            this.writeNiporlan(new NiporlanItem(object));
        }
    }
    
    private SyntaxData newSyntaxData() {
        SyntaxData syntaxData;
        if (!this.unUsedSyntaxDatas.empty()) {
            syntaxData = this.unUsedSyntaxDatas.pop();
        }
        else {
            syntaxData = new SyntaxData();
        }
        return syntaxData;
    }
    
    private void disposeSyntaxData(final SyntaxData syntaxData) {
        this.unUsedSyntaxDatas.push(syntaxData);
    }
    
    private void checkOpData(final SyntaxData opData) {
        if (opData.type == 9) {
            this.ret = this.getCellData(opData.cdCell, opData.value);
            if (this.ret != 0) {
                return;
            }
            opData.assign(opData.value);
        }
    }
    
    private void checkOpType(final SyntaxData lopData, final SyntaxData ropData, final int opCode) {
        if (lopData.type == 0 || ropData.type == 0) {
            lopData.type = 0;
            this.ret = 0;
            return;
        }
        if (opCode == 6) {
            if (lopData.type != 9 || lopData.type != ropData.type) {
                this.ret = 152;
                return;
            }
            if (!lopData.cdCell.tableName.equals(ropData.cdCell.tableName) || !lopData.cdCell.taskName.equals(ropData.cdCell.taskName) || lopData.cdCell.isZB || ropData.cdCell.isZB || lopData.cdCell.year != ropData.cdCell.year || lopData.cdCell.termKind != ropData.cdCell.termKind || lopData.cdCell.term != ropData.cdCell.term || (lopData.value.type != 1 && lopData.value.type != 2 && lopData.value.type != 10) || (ropData.value.type != 1 && ropData.value.type != 2 && ropData.value.type != 10)) {
                this.ret = 152;
                return;
            }
            if (lopData.cdCell.row > ropData.cdCell.row || lopData.cdCell.col > ropData.cdCell.col) {
                this.ret = 154;
                return;
            }
            final SyntaxData sumSyData = this.newSyntaxData();
            try {
                sumSyData.assign(lopData);
                this.tempSyData.assign(lopData);
                sumSyData.value.type = 2;
                sumSyData.value.cdReal = 0.0;
                for (int r = lopData.cdCell.row; r <= ropData.cdCell.row; ++r) {
                    int c = lopData.cdCell.col;
                    while (c <= ropData.cdCell.col) {
                        this.tempSyData.cdCell.row = r;
                        this.tempSyData.cdCell.col = c;
                        this.ret = this.getCellData(this.tempSyData.cdCell, this.tempSyData.value);
                        if (this.ret == 0) {
                            if (this.tempSyData.value.type == 2) {
                                sumSyData.value.cdReal += this.tempSyData.value.cdReal;
                            }
                            else {
                                if (this.tempSyData.value.type != 1) {
                                    this.ret = 152;
                                    return;
                                }
                                sumSyData.value.cdReal += this.tempSyData.value.cdInt;
                            }
                            ++c;
                            continue;
                        }
                        return;
                    }
                }
                lopData.type = 2;
                lopData.isNull = false;
                lopData.cdReal = sumSyData.value.cdReal;
                this.ret = 0;
                return;
            }
            finally {
                this.disposeSyntaxData(sumSyData);
            }
        }
        if (ropData.type == 9) {
            this.ret = this.getCellData(ropData.cdCell, ropData.value);
            if (this.ret != 0) {
                return;
            }
            ropData.assign(ropData.value);
            ropData.initNullValue();
        }
        if (opCode == 17) {
            if (lopData.type != 9) {
                this.ret = 152;
                return;
            }
            if (lopData.value.type != 0 && lopData.value.type != ropData.type && lopData.value.type != 1 && lopData.value.type != 2 && lopData.value.type != 10 && ropData.value.type != 1 && ropData.value.type != 2 && ropData.value.type != 10) {
                this.ret = 152;
                return;
            }
            lopData.value.assign(ropData);
            if (this.canAssignValue) {
                this.ret = this.setCellData(lopData.cdCell, lopData.value);
            }
        }
        else {
            if (lopData.type == 9) {
                this.ret = this.getCellData(lopData.cdCell, lopData.value);
                if (this.ret != 0) {
                    return;
                }
                lopData.assign(lopData.value);
                lopData.initNullValue();
            }
            if (opCode == 15) {
                if (lopData.type != 5 || ropData.type != 5) {
                    this.ret = 152;
                    return;
                }
                lopData.cdBool = (lopData.cdBool || ropData.cdBool);
                lopData.isNull = false;
                this.ret = 0;
            }
            else if (opCode == 14) {
                if (lopData.type != 5 || ropData.type != 5) {
                    this.ret = 152;
                    return;
                }
                lopData.cdBool = (lopData.cdBool && ropData.cdBool);
                lopData.isNull = false;
                this.ret = 0;
            }
            else if (opCode == 1 || opCode == 2 || opCode == 4 || opCode == 5) {
                if (lopData.type == 5) {
                    lopData.type = 1;
                    if (lopData.cdBool) {
                        lopData.cdInt = -1;
                    }
                    else {
                        lopData.cdInt = 0;
                    }
                }
                else if (ropData.type == 5) {
                    ropData.type = 1;
                    if (ropData.cdBool) {
                        ropData.cdInt = -1;
                    }
                    else {
                        ropData.cdInt = 0;
                    }
                }
                else if ((lopData.type == 3 && ropData.type != 3) || (ropData.type != 3 && lopData.type == 3)) {
                    this.ret = 152;
                    return;
                }
                if (lopData.type == 3) {
                    if (opCode == 1) {
                        lopData.cdString = String.valueOf(lopData.cdString) + ropData.cdString;
                        lopData.isNull = (lopData.isNull && ropData.isNull);
                        this.ret = 0;
                        return;
                    }
                    this.ret = 152;
                }
                else {
                    if ((lopData.type == 6 && ropData.type != 6) || (ropData.type != 6 && lopData.type == 6)) {
                        this.ret = 152;
                        return;
                    }
                    if (lopData.type == 6) {
                        if (opCode == 2) {
                            lopData.type = 1;
                            lopData.cdInt = minusDate(lopData.cdDate, ropData.cdDate);
                            lopData.isNull = false;
                            this.ret = 0;
                            return;
                        }
                        this.ret = 152;
                    }
                    else {
                        if (lopData.type == 1 && ropData.type == 1) {
                            switch (opCode) {
                                case 1: {
                                    if (Math.abs(lopData.cdInt + (double)ropData.cdInt) > 2.147483647E9) {
                                        lopData.type = 2;
                                        lopData.cdReal = lopData.cdInt + (double)ropData.cdInt;
                                        break;
                                    }
                                    lopData.cdInt += ropData.cdInt;
                                    break;
                                }
                                case 2: {
                                    if (Math.abs(lopData.cdInt - (double)ropData.cdInt) > 2.147483647E9) {
                                        lopData.type = 2;
                                        lopData.cdReal = lopData.cdInt - (double)ropData.cdInt;
                                        break;
                                    }
                                    lopData.cdInt -= ropData.cdInt;
                                    break;
                                }
                                case 4: {
                                    if (Math.abs(lopData.cdInt * (double)ropData.cdInt) > 2.147483647E9) {
                                        lopData.type = 2;
                                        lopData.cdReal = lopData.cdInt * (double)ropData.cdInt;
                                        break;
                                    }
                                    lopData.cdInt *= ropData.cdInt;
                                    break;
                                }
                                case 5: {
                                    lopData.type = 2;
                                    if (ropData.cdInt == 0) {
                                        lopData.cdReal = 0.0;
                                        break;
                                    }
                                    lopData.cdReal = lopData.cdInt / (double)ropData.cdInt;
                                    break;
                                }
                            }
                            lopData.isNull = false;
                            this.ret = 0;
                            return;
                        }
                        if ((lopData.type == 2 || lopData.type == 1) && (ropData.type == 2 || ropData.type == 1)) {
                            if (lopData.type == 1) {
                                lopData.type = 2;
                                lopData.cdReal = lopData.cdInt;
                            }
                            if (ropData.type == 1) {
                                ropData.type = 2;
                                ropData.cdReal = ropData.cdInt;
                            }
                            switch (opCode) {
                                case 1: {
                                    lopData.cdReal += ropData.cdReal;
                                    break;
                                }
                                case 2: {
                                    if (Math.abs(lopData.cdReal - ropData.cdReal) < 1.0E-5) {
                                        lopData.cdReal = 0.0;
                                        break;
                                    }
                                    lopData.cdReal -= ropData.cdReal;
                                    break;
                                }
                                case 4: {
                                    lopData.cdReal *= ropData.cdReal;
                                    break;
                                }
                                case 5: {
                                    if (ropData.cdReal == 0.0) {
                                        lopData.cdReal = 0.0;
                                        break;
                                    }
                                    lopData.cdReal /= ropData.cdReal;
                                    break;
                                }
                            }
                            lopData.isNull = false;
                            this.ret = 0;
                            return;
                        }
                        if ((lopData.type == 2 || lopData.type == 1 || lopData.type == 10) && (ropData.type == 2 || ropData.type == 1 || ropData.type == 10)) {
                            if (lopData.type != 10) {
                                lopData.type = 10;
                                if (lopData.type == 1) {
                                    lopData.cdDecimal = new BigDecimal(lopData.cdInt);
                                }
                                else if (lopData.type == 2) {
                                    lopData.cdDecimal = new BigDecimal(lopData.cdReal);
                                }
                            }
                            if (lopData.cdDecimal == null) {
                                lopData.cdDecimal = new BigDecimal(0);
                            }
                            if (ropData.type != 10) {
                                ropData.type = 10;
                                if (ropData.type == 1) {
                                    ropData.cdDecimal = new BigDecimal(ropData.cdInt);
                                }
                                else if (ropData.type == 2) {
                                    ropData.cdDecimal = new BigDecimal(ropData.cdReal);
                                }
                            }
                            if (ropData.cdDecimal == null) {
                                ropData.cdDecimal = new BigDecimal(0);
                            }
                            switch (opCode) {
                                case 1: {
                                    lopData.cdDecimal = lopData.cdDecimal.add(ropData.cdDecimal);
                                    break;
                                }
                                case 2: {
                                    lopData.cdDecimal = lopData.cdDecimal.subtract(ropData.cdDecimal);
                                    if (lopData.cdDecimal.compareTo(new BigDecimal(1.0E-5)) < 0) {
                                        lopData.cdDecimal = new BigDecimal(0);
                                        break;
                                    }
                                    break;
                                }
                                case 4: {
                                    lopData.cdDecimal = lopData.cdDecimal.multiply(ropData.cdDecimal);
                                    break;
                                }
                                case 5: {
                                    if (ropData.cdDecimal.compareTo(new BigDecimal(1.0E-5)) < 0) {
                                        lopData.cdDecimal = new BigDecimal(0);
                                        break;
                                    }
                                    lopData.cdDecimal = lopData.cdDecimal.divide(ropData.cdDecimal, 4);
                                    break;
                                }
                            }
                            lopData.isNull = false;
                            this.ret = 0;
                            return;
                        }
                        this.ret = 152;
                    }
                }
            }
            else if (opCode == 8) {
                if (lopData.type != 3 || ropData.type != 3) {
                    this.ret = 152;
                    return;
                }
                lopData.type = 5;
                lopData.isNull = false;
                this.ret = 0;
            }
            else {
                if (opCode != 7 && opCode != 9 && opCode != 10 && opCode != 11 && opCode != 12 && opCode != 13) {
                    return;
                }
                if ((lopData.type == 3 && ropData.type != 3) || (lopData.type != 3 && ropData.type == 3)) {
                    this.ret = 152;
                    return;
                }
                if ((lopData.type == 6 && ropData.type != 6) || (lopData.type != 6 && ropData.type == 6)) {
                    this.ret = 152;
                    return;
                }
                if (lopData.type == 3) {
                    lopData.type = 5;
                    switch (opCode) {
                        case 7: {
                            lopData.cdBool = lopData.cdString.equals(ropData.cdString);
                            break;
                        }
                        case 9: {
                            lopData.cdBool = !lopData.cdString.equals(ropData.cdString);
                            break;
                        }
                        case 10: {
                            lopData.cdBool = (lopData.cdString.compareTo(ropData.cdString) > 0);
                            break;
                        }
                        case 11: {
                            lopData.cdBool = (lopData.cdString.compareTo(ropData.cdString) < 0);
                            break;
                        }
                        case 12: {
                            lopData.cdBool = (lopData.cdString.compareTo(ropData.cdString) >= 0);
                            break;
                        }
                        case 13: {
                            lopData.cdBool = (lopData.cdString.compareTo(ropData.cdString) <= 0);
                            break;
                        }
                    }
                    lopData.isNull = false;
                    this.ret = 0;
                    return;
                }
                if (lopData.type == 6) {
                    lopData.getValidcdDate();
                    ropData.getValidcdDate();
                    lopData.type = 5;
                    switch (opCode) {
                        case 7: {
                            lopData.cdBool = lopData.cdDate.equals(ropData.cdDate);
                            break;
                        }
                        case 9: {
                            lopData.cdBool = !lopData.cdDate.equals(ropData.cdDate);
                            break;
                        }
                        case 10: {
                            lopData.cdBool = lopData.cdDate.after(ropData.cdDate);
                            break;
                        }
                        case 11: {
                            lopData.cdBool = lopData.cdDate.before(ropData.cdDate);
                            break;
                        }
                        case 12: {
                            lopData.cdBool = (lopData.cdDate.after(ropData.cdDate) || lopData.cdDate.equals(ropData.cdDate));
                            break;
                        }
                        case 13: {
                            lopData.cdBool = (lopData.cdDate.before(ropData.cdDate) || lopData.cdDate.equals(ropData.cdDate));
                            break;
                        }
                    }
                    lopData.isNull = false;
                    this.ret = 0;
                    return;
                }
                if (lopData.type == 1 && ropData.type == 1) {
                    lopData.type = 5;
                    switch (opCode) {
                        case 7: {
                            lopData.cdBool = (lopData.cdInt == ropData.cdInt);
                            break;
                        }
                        case 9: {
                            lopData.cdBool = (lopData.cdInt != ropData.cdInt);
                            break;
                        }
                        case 10: {
                            lopData.cdBool = (lopData.cdInt > ropData.cdInt);
                            break;
                        }
                        case 11: {
                            lopData.cdBool = (lopData.cdInt < ropData.cdInt);
                            break;
                        }
                        case 12: {
                            lopData.cdBool = (lopData.cdInt >= ropData.cdInt);
                            break;
                        }
                        case 13: {
                            lopData.cdBool = (lopData.cdInt <= ropData.cdInt);
                            break;
                        }
                    }
                    lopData.isNull = false;
                    this.ret = 0;
                    return;
                }
                if (lopData.type == 1) {
                    lopData.type = 2;
                    lopData.cdReal = lopData.cdInt;
                }
                if (ropData.type == 1) {
                    ropData.type = 2;
                    ropData.cdReal = ropData.cdInt;
                }
                lopData.type = 5;
                switch (opCode) {
                    case 7: {
                        lopData.cdBool = (Math.abs(lopData.cdReal - ropData.cdReal) < this.getBalanceDifferenceValue());
                        break;
                    }
                    case 9: {
                        lopData.cdBool = (Math.abs(lopData.cdReal - ropData.cdReal) >= this.getBalanceDifferenceValue());
                        break;
                    }
                    case 10: {
                        lopData.cdBool = (lopData.cdReal - ropData.cdReal > 1.0E-5);
                        break;
                    }
                    case 11: {
                        lopData.cdBool = (lopData.cdReal - ropData.cdReal < -1.0E-5);
                        break;
                    }
                    case 12: {
                        lopData.cdBool = (lopData.cdReal - ropData.cdReal > this.getBalanceDifferenceValue() || Math.abs(lopData.cdReal - ropData.cdReal) < this.getBalanceDifferenceValue());
                        break;
                    }
                    case 13: {
                        lopData.cdBool = (lopData.cdReal - ropData.cdReal < -this.getBalanceDifferenceValue() || Math.abs(lopData.cdReal - ropData.cdReal) < this.getBalanceDifferenceValue());
                        break;
                    }
                }
                lopData.isNull = false;
                this.ret = 0;
            }
        }
    }
    
    private void multorFuncParam(final FunctionParam param, final SyntaxData opData) {
        if (param.type == 3 && param.isSign) {
            if (this.ch == ' ') {
                this.skipSpaces();
            }
            String tmpStr = "";
            while (isSignChar(this.ch) || this.ch == '<' || this.ch == '>' || this.ch == ':' || this.ch == '#') {
                tmpStr = String.valueOf(tmpStr) + this.ch;
                this.getChar();
            }
            if (!tmpStr.equals("")) {
                if (this.hasExpDescription) {
                    this.addStringCaption(tmpStr);
                }
                opData.type = 3;
                opData.isNull = false;
                opData.cdString = tmpStr;
                if (this.hasMakeNiporlan) {
                    this.writeNiporlan(opData);
                }
            }
            else {
                this.wholeExp(opData);
                if (this.ret != 0) {
                    return;
                }
                this.tempSyData.type = 3;
                this.tempSyData.isNull = false;
                this.tempSyData.cdString = "";
                this.checkOpType(opData, this.tempSyData, 1);
                if (this.ret != 0) {
                    this.ret = 17;
                }
            }
        }
        else if (param.type == 4) {
            if (this.ch == ' ') {
                this.skipSpaces();
            }
            if (this.ch != '\'') {
                this.ret = 202;
                return;
            }
            this.getChar();
            this.tmpStrBuffer.setLength(0);
            while (this.ch != '\'' && this.ch != '\0') {
                this.tmpStrBuffer.append(this.ch);
                this.getChar();
            }
            if (this.ch != '\'') {
                this.ret = 202;
                return;
            }
            opData.type = 4;
            opData.isNull = false;
            opData.cdString = this.tmpStrBuffer.toString();
            if (this.hasMakeNiporlan) {
                this.writeNiporlan(opData);
            }
            this.getChar();
            if (this.hasExpDescription) {
                this.addStringCaption("'" + opData.cdString + "'");
            }
        }
        else {
            this.wholeExp(opData);
            if (this.ret != 0) {
                return;
            }
            if (param.type == 3) {
                this.tempSyData.type = 3;
                this.tempSyData.isNull = false;
                this.tempSyData.cdString = "";
                this.checkOpType(opData, this.tempSyData, 1);
                if (this.ret != 0) {
                    this.ret = 17;
                }
            }
            else if (param.type == 2 || param.type == 1) {
                this.tempSyData.type = 1;
                this.tempSyData.isNull = false;
                this.tempSyData.cdInt = 1;
                this.checkOpType(opData, this.tempSyData, 4);
                if (this.ret != 0) {
                    this.ret = 16;
                }
            }
            else if (param.type == 5) {
                this.tempSyData.type = 5;
                this.tempSyData.isNull = false;
                this.tempSyData.cdBool = false;
                this.checkOpType(opData, this.tempSyData, 15);
                if (this.ret != 0) {
                    this.ret = 19;
                }
            }
            else if (param.type == 6) {
                this.checkOpData(opData);
                if (this.ret != 0 || opData.type != 6) {
                    this.ret = 18;
                }
            }
            else if (param.type == 0) {
                this.checkOpData(opData);
                if (this.ret != 0) {
                    return;
                }
            }
        }
    }
    
    private void multor(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_17163: {
            try {
                if (this.ch == '\0') {
                    this.ret = 9;
                }
                else {
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    if (this.ch == '\"') {
                        this.tmpStrBuffer.setLength(0);
                        this.getChar();
                        while (this.ch != '\"' && this.ch != '\0') {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                        }
                        if (this.ch == '\"') {
                            this.getChar();
                        }
                        if (this.hasExpDescription) {
                            this.addStringCaption("\"" + (Object)this.tmpStrBuffer + "\"");
                        }
                        syData.type = 3;
                        syData.cdString = this.tmpStrBuffer.toString();
                        syData.isNull = false;
                        if (this.hasMakeNiporlan && !this.inParseCell && this.inEffectCheck) {
                            this.writeNiporlan(syData);
                        }
                        this.ret = 0;
                    }
                    else if ((this.ch >= '0' && this.ch <= '9') || (this.ch == '-' && this.getNextChar() >= '0' && this.getNextChar() <= '9')) {
                        if (this.hasExpDescription) {
                            this.addStringCaption(this.ch);
                        }
                        if (this.ch == '-') {
                            this.getChar();
                            this.tempReal = -Double.parseDouble(String.valueOf(this.ch));
                        }
                        else {
                            this.tempReal = Double.parseDouble(String.valueOf(this.ch));
                        }
                        this.getChar();
                        while (this.ch >= '0' && this.ch <= '9') {
                            if (this.hasExpDescription) {
                                this.addStringCaption(this.ch);
                            }
                            this.tempReal = this.tempReal * 10.0 + Double.parseDouble(String.valueOf(this.ch));
                            this.getChar();
                        }
                        if (this.ch == '.') {
                            if (this.hasExpDescription) {
                                this.addStringCaption(this.ch);
                            }
                            this.getChar();
                            this.decPos = 0.1;
                            while (this.ch >= '0' && this.ch <= '9') {
                                if (this.hasExpDescription) {
                                    this.addStringCaption(this.ch);
                                }
                                this.tempReal += Double.parseDouble(String.valueOf(this.ch)) * this.decPos;
                                this.decPos /= 10.0;
                                this.getChar();
                            }
                            syData.type = 2;
                            syData.cdReal = this.tempReal;
                            syData.isNull = false;
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(syData);
                            }
                            this.ret = 0;
                        }
                        else if (this.tempReal < 2.147483647E9) {
                            syData.type = 1;
                            syData.cdInt = (int)this.tempReal;
                            syData.isNull = false;
                            if (this.hasMakeNiporlan && !this.inParseCell && this.inEffectCheck) {
                                this.writeNiporlan(syData);
                            }
                            this.ret = 0;
                        }
                        else {
                            syData.type = 2;
                            syData.cdReal = this.tempReal;
                            syData.isNull = false;
                            if (this.hasMakeNiporlan && !this.inParseCell && this.inEffectCheck) {
                                this.writeNiporlan(syData);
                            }
                            this.ret = 0;
                        }
                    }
                    else if (this.ch == '(') {
                        if (this.hasExpDescription) {
                            this.addStringCaption(this.ch);
                        }
                        this.getChar();
                        this.wholeExp(syData);
                        if (this.ret == 0) {
                            if (this.ch == ' ') {
                                this.skipSpaces();
                            }
                            if (this.ch != ')') {
                                this.ret = 3;
                            }
                            else {
                                if (this.hasExpDescription) {
                                    this.addStringCaption(this.ch);
                                }
                                this.getChar();
                                this.ret = 0;
                            }
                        }
                    }
                    else {
                        if (this.ch != '[' || !this.hasCellSyntax) {
                            if (isFirstSignChar(this.ch)) {
                                this.tmpStrBuffer.setLength(0);
                                this.tmpStrBuffer.append(this.ch);
                                this.getChar();
                                while (isSignChar(this.ch) || this.ch == '$') {
                                    this.tmpStrBuffer.append(this.ch);
                                    this.getChar();
                                }
                                this.tmpStr = this.tmpStrBuffer.toString();
                                if (SyntaxParser.RESERVED_SIGNS.containsKey(this.tmpStr)) {
                                    if (this.tmpStr.equals("L$")) {
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != '(') {
                                            this.ret = 2;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption("L$(");
                                        }
                                        this.wholeExp(syData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        this.tempSyData.type = 3;
                                        this.tempSyData.cdString = "";
                                        this.checkOpType(syData, this.tempSyData, 1);
                                        if (this.ret != 0) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (syData.type != 3) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ',') {
                                            this.ret = 14;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(",");
                                        }
                                        this.getChar();
                                        this.wholeExp(leftOpData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        if (leftOpData.type != 1) {
                                            this.ret = 15;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ')') {
                                            this.ret = 3;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(")");
                                        }
                                        if (this.hasMakeNiporlan) {
                                            this.writeNiporlan("L$");
                                        }
                                        this.getChar();
                                        syData.cdString = asciiCopy(syData.cdString, 1, leftOpData.cdInt);
                                        this.ret = 0;
                                        return;
                                    }
                                    else if (this.tmpStr.equals("M$")) {
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != '(') {
                                            this.ret = 2;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption("M$(");
                                        }
                                        this.wholeExp(syData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        this.tempSyData.type = 3;
                                        this.tempSyData.cdString = "";
                                        this.checkOpType(syData, this.tempSyData, 1);
                                        if (this.ret != 0) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (syData.type != 3) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ',') {
                                            this.ret = 14;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(",");
                                        }
                                        this.getChar();
                                        this.wholeExp(leftOpData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        if (leftOpData.type != 1) {
                                            this.ret = 15;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ',') {
                                            this.ret = 14;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(",");
                                        }
                                        this.getChar();
                                        this.wholeExp(rightOpData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        if (rightOpData.type != 1) {
                                            this.ret = 15;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ')') {
                                            this.ret = 3;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(")");
                                        }
                                        if (this.hasMakeNiporlan) {
                                            this.writeNiporlan("M$");
                                        }
                                        syData.cdString = asciiCopy(syData.cdString, leftOpData.cdInt, rightOpData.cdInt);
                                        this.ret = 0;
                                        return;
                                    }
                                    else if (this.tmpStr.equals("R$")) {
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != '(') {
                                            this.ret = 2;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption("R$(");
                                        }
                                        this.wholeExp(syData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        this.tempSyData.type = 3;
                                        this.tempSyData.cdString = "";
                                        this.checkOpType(syData, this.tempSyData, 1);
                                        if (this.ret != 0) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (syData.type != 3) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ',') {
                                            this.ret = 14;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(",");
                                        }
                                        this.getChar();
                                        this.wholeExp(leftOpData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        if (leftOpData.type != 1) {
                                            this.ret = 15;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ')') {
                                            this.ret = 3;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(")");
                                        }
                                        if (this.hasMakeNiporlan) {
                                            this.writeNiporlan("R$");
                                        }
                                        if (asciiLength(syData.cdString) + 1 - leftOpData.cdInt >= 1) {
                                            syData.cdString = asciiCopy(syData.cdString, asciiLength(syData.cdString) + 1 - leftOpData.cdInt, leftOpData.cdInt);
                                        }
                                        else {
                                            syData.cdString = syData.cdString;
                                        }
                                        this.ret = 0;
                                        return;
                                    }
                                    else if (this.tmpStr.equals("POS")) {
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != '(') {
                                            this.ret = 2;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption("字符位置(");
                                        }
                                        this.wholeExp(syData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        this.tempSyData.type = 3;
                                        this.tempSyData.cdString = "";
                                        this.checkOpType(syData, this.tempSyData, 1);
                                        if (this.ret != 0 || syData.type != 3) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ',') {
                                            this.ret = 14;
                                            return;
                                        }
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(",");
                                        }
                                        this.getChar();
                                        this.wholeExp(leftOpData);
                                        if (this.ret != 0) {
                                            return;
                                        }
                                        this.tempSyData.type = 3;
                                        this.tempSyData.cdString = "";
                                        this.checkOpType(leftOpData, this.tempSyData, 1);
                                        if (this.ret != 0 || leftOpData.type != 3) {
                                            this.ret = 17;
                                            return;
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ')') {
                                            this.ret = 3;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(")");
                                        }
                                        if (this.hasMakeNiporlan) {
                                            this.writeNiporlan("POS");
                                        }
                                        syData.type = 1;
                                        syData.cdInt = asciiPos(syData.cdString, leftOpData.cdString);
                                        this.ret = 0;
                                        return;
                                    }
                                    else {
                                        if (this.tmpStr.equals("T") || this.tmpStr.equals("TRUE")) {
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("真");
                                            }
                                            syData.type = 5;
                                            syData.cdBool = true;
                                            syData.isNull = false;
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(syData);
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        if (this.tmpStr.equals("F") || this.tmpStr.equals("FALSE")) {
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("假");
                                            }
                                            syData.type = 5;
                                            syData.cdBool = false;
                                            syData.isNull = false;
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(syData);
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        if (this.tmpStr.equals("NOT")) {
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("非 ");
                                            }
                                            this.mainItem(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (syData.type != 5) {
                                                this.ret = 152;
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(16);
                                            }
                                            syData.cdBool = !syData.cdBool;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("ABS")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("绝对值(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("ABS");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdInt = Math.abs(syData.cdInt);
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = Math.abs(syData.cdReal);
                                                    break;
                                                }
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("INT")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取整(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("INT");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdInt = syData.cdInt;
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = (double)trunc(syData.cdReal);
                                                    break;
                                                }
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("ROUND")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("四舍五入取整(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.tempSyData.isNull = false;
                                            this.checkOpType(syData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                this.multor(leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                if (leftOpData.type != 1) {
                                                    this.ret = 15;
                                                    return;
                                                }
                                            }
                                            else {
                                                leftOpData.type = 1;
                                                leftOpData.cdInt = 0;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(leftOpData);
                                                }
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("ROUND");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdInt = (int)Math.round(roundFloat(syData.cdInt, leftOpData.cdInt));
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = roundFloat(syData.cdReal, leftOpData.cdInt);
                                                    break;
                                                }
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("MOD")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取模(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.tempSyData.isNull = false;
                                            this.checkOpType(syData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            this.wholeExp(leftOpData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(leftOpData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("MOD");
                                            }
                                            if (syData.type == 1) {
                                                syData.type = 2;
                                                syData.cdReal = syData.cdInt;
                                            }
                                            if (leftOpData.type == 1) {
                                                leftOpData.type = 2;
                                                leftOpData.cdReal = leftOpData.cdInt;
                                            }
                                            syData.cdReal = floatMod(syData.cdReal, leftOpData.cdReal);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("POWER")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("乘方(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(syData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            this.wholeExp(leftOpData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(leftOpData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("POWER");
                                            }
                                            if (syData.type == 1) {
                                                syData.type = 2;
                                                syData.cdReal = syData.cdInt;
                                            }
                                            if (leftOpData.type == 1) {
                                                leftOpData.type = 2;
                                                leftOpData.cdReal = leftOpData.cdInt;
                                            }
                                            if (syData.cdReal < 0.0 && Math.abs(leftOpData.cdReal - trunc(leftOpData.cdReal)) > 1.0E-5) {
                                                syData.cdReal = 1.0;
                                            }
                                            else {
                                                syData.cdReal = Math.pow(syData.cdReal, leftOpData.cdReal);
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("TAN")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("正切值(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("TAN");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdReal = Math.tan(syData.cdInt);
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = Math.tan(syData.cdReal);
                                                    break;
                                                }
                                            }
                                            syData.type = 2;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("ATAN")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("反正切值(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("ATAN");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdReal = Math.atan(syData.cdInt);
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = Math.atan(syData.cdReal);
                                                    break;
                                                }
                                            }
                                            syData.type = 2;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("LN")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("自然对数(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("LN");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdReal = Math.log(syData.cdInt);
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = Math.log(syData.cdReal);
                                                    break;
                                                }
                                            }
                                            syData.type = 2;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("LOG10")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("10为底的对数(");
                                            }
                                            this.getChar();
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            leftOpData.type = 1;
                                            leftOpData.cdInt = 1;
                                            this.checkOpType(syData, leftOpData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("LOG10");
                                            }
                                            switch (syData.type) {
                                                case 1: {
                                                    syData.cdReal = Math.log(syData.cdInt) / Math.log(10.0);
                                                    break;
                                                }
                                                case 2: {
                                                    syData.cdReal = Math.log(syData.cdReal) / Math.log(10.0);
                                                    break;
                                                }
                                            }
                                            syData.type = 2;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("LOG")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("对数(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(syData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            this.wholeExp(leftOpData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(leftOpData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("LOG");
                                            }
                                            if (syData.type == 1) {
                                                syData.type = 2;
                                                syData.cdReal = syData.cdInt;
                                            }
                                            if (leftOpData.type == 1) {
                                                leftOpData.type = 2;
                                                leftOpData.cdReal = leftOpData.cdInt;
                                            }
                                            syData.cdReal = Math.log(syData.cdReal) / Math.log(leftOpData.cdReal);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("VAL")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("求值(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("VAL");
                                            }
                                            syData.type = 2;
                                            syData.cdReal = safeStrToFloat(syData.cdString);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("STR")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("转字符(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (syData.type == 9) {
                                                this.ret = this.getCellData(syData.cdCell, syData.value);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                syData.assign(syData.value);
                                                syData.initNullValue();
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (syData.type != 1 && syData.type != 2) {
                                                    this.ret = 3;
                                                    return;
                                                }
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.wholeExp(leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.tempSyData.type = 1;
                                                this.tempSyData.cdInt = 1;
                                                this.checkOpType(leftOpData, this.tempSyData, 4);
                                                if (this.ret != 0) {
                                                    this.ret = 15;
                                                    return;
                                                }
                                                if (leftOpData.type != 1 && leftOpData.type != 2) {
                                                    this.ret = 15;
                                                    return;
                                                }
                                                if (leftOpData.type == 2) {
                                                    this.tmpInt = (int)leftOpData.cdReal;
                                                }
                                                else {
                                                    this.tmpInt = leftOpData.cdInt;
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == ',') {
                                                    this.getChar();
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(",");
                                                    }
                                                    this.wholeExp(leftOpData);
                                                    if (this.ret != 0) {
                                                        return;
                                                    }
                                                    if (leftOpData.type != 5) {
                                                        this.ret = 19;
                                                        return;
                                                    }
                                                }
                                                else {
                                                    this.tempSyData.type = 5;
                                                    this.tempSyData.cdBool = false;
                                                    if (this.hasMakeNiporlan) {
                                                        this.writeNiporlan(this.tempSyData);
                                                    }
                                                    leftOpData.cdBool = false;
                                                }
                                            }
                                            else {
                                                this.tempSyData.type = 1;
                                                this.tempSyData.cdInt = 0;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                this.tmpInt = 0;
                                                this.tempSyData.type = 5;
                                                this.tempSyData.cdBool = false;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                leftOpData.cdBool = false;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("STR");
                                            }
                                            if (syData.type == 1) {
                                                syData.type = 3;
                                                syData.cdString = FloatToStr(syData.cdInt, this.tmpInt, leftOpData.cdBool);
                                            }
                                            else if (syData.type == 2) {
                                                syData.type = 3;
                                                syData.cdString = FloatToStr(syData.cdReal, this.tmpInt, leftOpData.cdBool);
                                            }
                                            else {
                                                syData.cdString = syData.toString();
                                                syData.type = 3;
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("IDC")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("校验位(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("IDC");
                                            }
                                            syData.cdString = String.valueOf(idc(syData.cdString));
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("ISDIGIT")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("是数字(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("ISDIGIT");
                                            }
                                            syData.type = 5;
                                            syData.cdBool = isDigit(syData.cdString);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("LEN")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("长度(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("LEN");
                                            }
                                            syData.type = 1;
                                            syData.cdInt = asciiLength(syData.cdString);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("ISNULL")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("是空值(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (syData.type == 9) {
                                                this.ret = this.getCellData(syData.cdCell, syData.value);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                syData.assign(syData.value);
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("ISNULL");
                                            }
                                            syData.type = 5;
                                            syData.cdBool = syData.isNull;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("EXISTCHINESE")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("有中文(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("EXISTCHINESE");
                                            }
                                            syData.type = 5;
                                            syData.cdBool = existChinese(syData.cdString);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("INLIST")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("在列表中(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (isFirstSignChar(this.ch)) {
                                                this.tmpStrBuffer.setLength(0);
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                                while (isSignChar(this.ch)) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                this.tmpStr = this.tmpStrBuffer.toString();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(this.tmpStr);
                                                }
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = this.tmpStr;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                final DictMeaningEvent event = new DictMeaningEvent(this);
                                                event.code = syData.cdString;
                                                event.dictName = this.tmpStr;
                                                event.meaningName = "";
                                                this.ret = this.getInDict(event);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                syData.type = 5;
                                                syData.cdBool = event.inDict;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan("INDICT");
                                                }
                                            }
                                            else {
                                                syData.type = 5;
                                                syData.cdBool = false;
                                                final CommonData count = new CommonData();
                                                count.type = 1;
                                                count.cdInt = 0;
                                                do {
                                                    if (this.ch == ',') {
                                                        this.getChar();
                                                        if (this.hasExpDescription) {
                                                            this.addStringCaption(",");
                                                        }
                                                    }
                                                    this.wholeExp(leftOpData);
                                                    if (this.ret != 0) {
                                                        return;
                                                    }
                                                    if (leftOpData.type != 3) {
                                                        this.ret = 17;
                                                        return;
                                                    }
                                                    if (syData.cdString.equals(leftOpData.cdString) || (!leftOpData.cdString.equals("") && syData.cdString.indexOf(leftOpData.cdString) == 0)) {
                                                        syData.cdBool = true;
                                                    }
                                                    final CommonData commonData = count;
                                                    ++commonData.cdInt;
                                                    if (this.ch != ' ') {
                                                        continue;
                                                    }
                                                    this.skipSpaces();
                                                } while (this.ch == ',');
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(count);
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan("INLIST");
                                                }
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("DICTVALUE") || this.tmpStr.equals("DV") || this.tmpStr.equals("GETMEANING")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取字典值(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            this.tmpStrBuffer.setLength(0);
                                            this.tmpStrBuffer.append(this.ch);
                                            this.getChar();
                                            while (isSignChar(this.ch)) {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            this.tmpStr = this.tmpStrBuffer.toString();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(this.tmpStr);
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = this.tmpStr;
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(this.tempSyData);
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                this.tmpStrBuffer.setLength(0);
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                                while (isSignChar(this.ch)) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                this.tmpStr2 = this.tmpStrBuffer.toString();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(this.tmpStr2);
                                                }
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = this.tmpStr2;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                            }
                                            else {
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                            }
                                            final DictMeaningEvent event = new DictMeaningEvent(this);
                                            event.code = syData.cdString;
                                            event.dictName = this.tmpStr;
                                            event.meaningName = this.tempSyData.cdString;
                                            this.ret = this.getDictValue(event);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            syData.assign(event.meaningValue);
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("DICTVALUE");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("STATE")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("单位状态(");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.wholeExp(syData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                this.checkOpType(syData, this.tempSyData, 1);
                                                if (this.ret != 0) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                                if (syData.type != 3) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == ',') {
                                                    this.getChar();
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(",");
                                                    }
                                                    this.wholeExp(leftOpData);
                                                    if (this.ret != 0) {
                                                        return;
                                                    }
                                                    this.tempSyData.type = 3;
                                                    this.tempSyData.cdString = "";
                                                    this.checkOpType(leftOpData, this.tempSyData, 1);
                                                    if (this.ret != 0) {
                                                        this.ret = 17;
                                                        return;
                                                    }
                                                    if (leftOpData.type != 3) {
                                                        this.ret = 17;
                                                        return;
                                                    }
                                                }
                                                else {
                                                    this.tempSyData.type = 3;
                                                    this.tempSyData.cdString = "";
                                                    if (this.hasMakeNiporlan) {
                                                        this.writeNiporlan(this.tempSyData);
                                                    }
                                                    leftOpData.cdString = "";
                                                }
                                            }
                                            else {
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                this.tempSyData.isNull = false;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                syData.cdString = "";
                                                leftOpData.cdString = "";
                                            }
                                            final UnitStateEvent event2 = new UnitStateEvent(this, syData.cdString, leftOpData.cdString);
                                            this.ret = this.getUnitState(event2);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            syData.isNull = false;
                                            syData.type = 1;
                                            syData.cdInt = event2.state;
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("UNITSTATE");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("FLOATSTAT")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("浮动行统计(");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            final FloatStatEvent event3 = new FloatStatEvent(this);
                                            this.getChar();
                                            this.tmpStrBuffer.setLength(0);
                                            while (this.ch != '\'' && this.ch != '\0') {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            event3.expression = this.tmpStrBuffer.toString();
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("'" + event3.expression + "'");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch != '\'') {
                                                    this.ret = 202;
                                                    return;
                                                }
                                                this.getChar();
                                                this.tmpStrBuffer.setLength(0);
                                                while (this.ch != '\'' && this.ch != '\0') {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                if (this.ch != '\'') {
                                                    this.ret = 202;
                                                    return;
                                                }
                                                event3.condition = this.tmpStrBuffer.toString();
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption("'" + event3.condition + "'");
                                                }
                                            }
                                            else {
                                                event3.condition = "";
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                this.tmpStrBuffer.setLength(0);
                                                while (isLetterChar(this.ch) || isChinaChar(this.ch)) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                this.tmpStr = this.tmpStrBuffer.toString();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (DataCell.statSignToInt(this.tmpStr) < 0) {
                                                    this.ret = 202;
                                                    return;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(this.tmpStr);
                                                }
                                                event3.statMode = DataCell.statSignToInt(this.tmpStr);
                                            }
                                            else {
                                                event3.statMode = 1;
                                            }
                                            this.ret = this.getFloatStat(event3);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            syData.assign(event3.value);
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(event3);
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("FLOATSTAT");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("FLOATCOPY")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("浮动行Copy(");
                                            }
                                            final FloatCopyEvent event4 = new FloatCopyEvent(this);
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            this.getChar();
                                            this.tmpStrBuffer.setLength(0);
                                            while (this.ch != '\'' && this.ch != '\0') {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            event4.sourceRelaExp = this.tmpStrBuffer.toString();
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("'" + event4.sourceRelaExp + "'");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            this.getChar();
                                            this.tmpStrBuffer.setLength(0);
                                            while (this.ch != '\'' && this.ch != '\0') {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            event4.destRelaExp = this.tmpStrBuffer.toString();
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("'" + event4.destRelaExp + "'");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            this.getChar();
                                            this.tmpStrBuffer.setLength(0);
                                            while (this.ch != '\'' && this.ch != '\0') {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            event4.sourceFilter = this.tmpStrBuffer.toString();
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("'" + event4.sourceFilter + "'");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            this.getChar();
                                            this.tmpStrBuffer.setLength(0);
                                            while (this.ch != '\'' && this.ch != '\0') {
                                                this.tmpStrBuffer.append(this.ch);
                                                this.getChar();
                                            }
                                            if (this.ch != '\'') {
                                                this.ret = 202;
                                                return;
                                            }
                                            event4.calcExps = this.tmpStrBuffer.toString();
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("'" + event4.calcExps + "'");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                this.tmpStrBuffer.setLength(0);
                                                while (isSignChar(this.ch)) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                this.tmpStr = this.tmpStrBuffer.toString();
                                                if (this.tmpStr.equalsIgnoreCase("false") || this.tmpStr.equalsIgnoreCase("0")) {
                                                    event4.isClearData = false;
                                                }
                                                else {
                                                    if (!this.tmpStr.equalsIgnoreCase("true") && !this.tmpStr.equalsIgnoreCase("1")) {
                                                        this.ret = 202;
                                                        return;
                                                    }
                                                    event4.isClearData = true;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(this.tmpStr);
                                                }
                                            }
                                            else {
                                                event4.isClearData = false;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                this.tmpStrBuffer.setLength(0);
                                                while (isSignChar(this.ch)) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    this.getChar();
                                                }
                                                this.tmpStr = this.tmpStrBuffer.toString();
                                                if (this.tmpStr.equalsIgnoreCase("false") || this.tmpStr.equalsIgnoreCase("0")) {
                                                    event4.copyType = 0;
                                                }
                                                else if (this.tmpStr.equalsIgnoreCase("true") || this.tmpStr.equalsIgnoreCase("1")) {
                                                    event4.copyType = 1;
                                                }
                                                else if (this.tmpStr.equals("2")) {
                                                    event4.copyType = 2;
                                                }
                                                else {
                                                    try {
                                                        final int value = Integer.parseInt(this.tmpStr);
                                                        if (value < 0 && value > 100) {
                                                            this.ret = 202;
                                                            return;
                                                        }
                                                        event4.copyType = value;
                                                    }
                                                    catch (final NumberFormatException ex) {
                                                        this.ret = 202;
                                                        return;
                                                    }
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(this.tmpStr);
                                                }
                                            }
                                            else {
                                                event4.copyType = 0;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                this.getChar();
                                                event4.timeOffset = this.getInteger();
                                            }
                                            event4.srcPeriodType = '@';
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == '\'') {
                                                    this.getChar();
                                                    this.tmpStrBuffer.setLength(0);
                                                    while (this.ch != '\'' && this.ch != '\0') {
                                                        this.tmpStrBuffer.append(this.ch);
                                                        this.getChar();
                                                    }
                                                    if (this.ch != '\'') {
                                                        this.ret = 202;
                                                        return;
                                                    }
                                                    event4.srcPeriodType = this.tmpStrBuffer.toString().charAt(0);
                                                    this.getChar();
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption("'" + event4.srcPeriodType + "'");
                                                    }
                                                }
                                            }
                                            event4.periodCount = -1;
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                this.getChar();
                                                event4.periodCount = this.getInteger();
                                            }
                                            this.ret = this.execFloatCopy(event4);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            syData.type = 1;
                                            syData.cdInt = event4.copyRowCount;
                                            syData.isNull = false;
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan(event4);
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("FLOATCOPY");
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("DATEVALUE")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("日期(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 3;
                                            this.tempSyData.cdString = "";
                                            this.checkOpType(syData, this.tempSyData, 1);
                                            if (this.ret != 0) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (syData.type != 3) {
                                                this.ret = 17;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.wholeExp(leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                this.checkOpType(leftOpData, this.tempSyData, 1);
                                                if (this.ret != 0) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                                if (leftOpData.type != 3) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                            }
                                            else {
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                leftOpData.cdString = "";
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("DATEVALUE");
                                            }
                                            syData.type = 6;
                                            syData.isNull = !safeStrToDate(syData.getValidcdDate(), syData.cdString, leftOpData.cdString);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("FORMATDATE")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("格式化日期(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.getValidcdDate();
                                            this.tempSyData.type = 6;
                                            this.tempSyData.cdDate.clear();
                                            this.checkOpType(this.tempSyData, syData, 7);
                                            if (this.ret != 0) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (syData.type != 6) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch == ',') {
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                this.wholeExp(leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                this.checkOpType(leftOpData, this.tempSyData, 1);
                                                if (this.ret != 0) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                                if (leftOpData.type != 3) {
                                                    this.ret = 17;
                                                    return;
                                                }
                                            }
                                            else {
                                                this.tempSyData.type = 3;
                                                this.tempSyData.cdString = "";
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(this.tempSyData);
                                                }
                                                leftOpData.cdString = "";
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("FORMATDATE");
                                            }
                                            syData.cdString = formatDate(syData.getValidcdDate(), leftOpData.cdString);
                                            syData.type = 3;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("DATE")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("日期(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(syData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (syData.type == 2) {
                                                syData.type = 1;
                                                syData.cdInt = (int)trunc(syData.cdReal);
                                            }
                                            if (syData.type != 1) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            this.wholeExp(leftOpData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(leftOpData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (leftOpData.type == 2) {
                                                leftOpData.type = 1;
                                                leftOpData.cdInt = (int)trunc(leftOpData.cdReal);
                                            }
                                            if (leftOpData.type != 1) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',') {
                                                this.ret = 14;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(",");
                                            }
                                            this.getChar();
                                            this.wholeExp(rightOpData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.type = 1;
                                            this.tempSyData.cdInt = 1;
                                            this.checkOpType(rightOpData, this.tempSyData, 4);
                                            if (this.ret != 0) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (rightOpData.type == 2) {
                                                rightOpData.type = 1;
                                                rightOpData.cdInt = (int)trunc(rightOpData.cdReal);
                                            }
                                            if (rightOpData.type != 1) {
                                                this.ret = 15;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("DATE");
                                            }
                                            syData.type = 6;
                                            safeEncodeDate(syData.getValidcdDate(), syData.cdInt, leftOpData.cdInt, rightOpData.cdInt);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("YEAR")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取年(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.getValidcdDate();
                                            this.tempSyData.type = 6;
                                            this.tempSyData.cdDate.clear();
                                            this.checkOpType(this.tempSyData, syData, 7);
                                            if (this.ret != 0) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (syData.type != 6) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("YEAR");
                                            }
                                            syData.type = 1;
                                            syData.cdInt = syData.getValidcdDate().get(1);
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("MONTH")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取月(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.getValidcdDate();
                                            this.tempSyData.type = 6;
                                            this.tempSyData.cdDate.clear();
                                            this.checkOpType(this.tempSyData, syData, 7);
                                            if (this.ret != 0) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (syData.type != 6) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("MONTH");
                                            }
                                            syData.type = 1;
                                            syData.cdInt = syData.getValidcdDate().get(2) + 1;
                                            this.ret = 0;
                                            return;
                                        }
                                        else if (this.tmpStr.equals("DAY")) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '(') {
                                                this.ret = 2;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption("取日(");
                                            }
                                            this.wholeExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            this.tempSyData.getValidcdDate();
                                            this.tempSyData.type = 6;
                                            this.tempSyData.cdDate.clear();
                                            this.checkOpType(this.tempSyData, syData, 7);
                                            if (this.ret != 0) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (syData.type != 6) {
                                                this.ret = 18;
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ')') {
                                                this.ret = 3;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(")");
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.writeNiporlan("DAY");
                                            }
                                            syData.type = 1;
                                            syData.cdInt = syData.getValidcdDate().get(5);
                                            this.ret = 0;
                                            return;
                                        }
                                        else {
                                            if (this.tmpStr.equals("TODAY")) {
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == '(') {
                                                    this.getChar();
                                                    if (this.ch == ' ') {
                                                        this.skipSpaces();
                                                    }
                                                    if (this.ch != ')') {
                                                        this.ret = 3;
                                                        return;
                                                    }
                                                    this.getChar();
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption("今天");
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan("TODAY");
                                                }
                                                syData.type = 6;
                                                syData.cdDate = new GregorianCalendar();
                                                syData.cdDate.getTime().setTime(0L);
                                                this.ret = 0;
                                                return;
                                            }
                                            if (this.tmpStr.equals("NULLBLOB")) {
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == '(') {
                                                    this.getChar();
                                                    if (this.ch == ' ') {
                                                        this.skipSpaces();
                                                    }
                                                    if (this.ch != ')') {
                                                        this.ret = 3;
                                                        return;
                                                    }
                                                    this.getChar();
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption("空二进制");
                                                }
                                                syData.type = 8;
                                                syData.isNull = true;
                                                syData.cdObject = null;
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(syData);
                                                }
                                                this.ret = 0;
                                                return;
                                            }
                                            if (this.tmpStr.equals("LINKPLUS")) {
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch != '(') {
                                                    this.ret = 2;
                                                    return;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(String.valueOf(this.tmpStr) + "(");
                                                }
                                                this.getChar();
                                                this.parseCellExp(syData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addCellCaption(syData.cdCell);
                                                }
                                                if (this.hasExpVision) {
                                                    this.addCellVision(syData.cdCell);
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(syData.cdCell);
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch != ',') {
                                                    this.ret = 14;
                                                    return;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(",");
                                                }
                                                this.getChar();
                                                this.wholeExp(leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.tempSyData.type = 1;
                                                this.tempSyData.cdInt = 1;
                                                this.checkOpType(leftOpData, this.tempSyData, 4);
                                                if (this.ret != 0) {
                                                    this.ret = 15;
                                                    return;
                                                }
                                                if (leftOpData.type == 2) {
                                                    leftOpData.type = 1;
                                                    leftOpData.cdInt = (int)trunc(leftOpData.cdReal);
                                                }
                                                if (leftOpData.type != 1) {
                                                    this.ret = 15;
                                                    return;
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == ',') {
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(",");
                                                    }
                                                    this.getChar();
                                                    this.wholeExp(rightOpData);
                                                    if (this.ret != 0) {
                                                        return;
                                                    }
                                                    this.tempSyData.type = 1;
                                                    this.tempSyData.cdInt = 1;
                                                    this.checkOpType(rightOpData, this.tempSyData, 4);
                                                    if (this.ret != 0) {
                                                        this.ret = 15;
                                                        return;
                                                    }
                                                    if (rightOpData.type == 2) {
                                                        rightOpData.type = 1;
                                                        rightOpData.cdInt = (int)trunc(rightOpData.cdReal);
                                                    }
                                                    if (rightOpData.type != 1) {
                                                        this.ret = 15;
                                                        return;
                                                    }
                                                }
                                                else {
                                                    rightOpData.type = 1;
                                                    rightOpData.cdInt = 0;
                                                    if (this.hasMakeNiporlan) {
                                                        this.writeNiporlan(rightOpData);
                                                    }
                                                }
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch != ')') {
                                                    this.ret = 3;
                                                    return;
                                                }
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(")");
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan("LINKPLUS");
                                                }
                                                final LinkPlusEvent event5 = new LinkPlusEvent(this, syData.cdCell, leftOpData.cdInt, rightOpData.cdInt, syData.value);
                                                this.ret = this.getLinkPlus(event5);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                syData.assign(syData.value);
                                                this.ret = 0;
                                                return;
                                            }
                                            else if (this.tmpStr.equals("AUTOSUM") || this.tmpStr.equals("SUM") || this.tmpStr.equals("MAX") || this.tmpStr.equals("MIN") || this.tmpStr.equals("AVG") || this.tmpStr.equals("COUNT") || this.tmpStr.equals("FIRST") || this.tmpStr.equals("LAST")) {
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch != '(') {
                                                    this.ret = 2;
                                                    return;
                                                }
                                                syData.type = 9;
                                                syData.cdCell.init();
                                                syData.cdCell.statMode = DataCell.statSignToInt(this.tmpStr);
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(String.valueOf(this.tmpStr) + "(");
                                                }
                                                this.tmpStrBuffer.setLength(0);
                                                this.getChar();
                                                this.tmpInt = 0;
                                                this.tmpBool = false;
                                                int midBracket = 0;
                                                while (this.tmpInt > 0 || midBracket > 0 || this.tmpBool || (this.ch != ')' && this.ch != ',')) {
                                                    this.tmpStrBuffer.append(this.ch);
                                                    if (this.ch == '\"') {
                                                        this.tmpBool = !this.tmpBool;
                                                    }
                                                    if (!this.tmpBool) {
                                                        if (this.ch == '(') {
                                                            ++this.tmpInt;
                                                        }
                                                        if (this.ch == ')') {
                                                            --this.tmpInt;
                                                        }
                                                        if (this.ch == '[') {
                                                            ++midBracket;
                                                        }
                                                        if (this.ch == ']') {
                                                            --midBracket;
                                                        }
                                                    }
                                                    this.getChar();
                                                    if (this.ch == '\0') {
                                                        break;
                                                    }
                                                }
                                                syData.cdCell.zbName = this.tmpStrBuffer.toString();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(syData.cdCell.zbName);
                                                }
                                                if (this.ch == ',') {
                                                    this.tmpStrBuffer.setLength(0);
                                                    this.getChar();
                                                    this.tmpInt = 0;
                                                    this.tmpBool = false;
                                                    midBracket = 0;
                                                    while (this.tmpInt > 0 || midBracket > 0 || this.tmpBool || this.ch != ')') {
                                                        this.tmpStrBuffer.append(this.ch);
                                                        if (this.ch == '\"') {
                                                            this.tmpBool = !this.tmpBool;
                                                        }
                                                        if (!this.tmpBool) {
                                                            if (this.ch == '(') {
                                                                ++this.tmpInt;
                                                            }
                                                            if (this.ch == ')') {
                                                                --this.tmpInt;
                                                            }
                                                            if (this.ch == '[') {
                                                                ++midBracket;
                                                            }
                                                            if (this.ch == ']') {
                                                                --midBracket;
                                                            }
                                                        }
                                                        this.getChar();
                                                        if (this.ch == '\0') {
                                                            break;
                                                        }
                                                    }
                                                    syData.cdCell.condition = this.tmpStrBuffer.toString();
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(",");
                                                    }
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(syData.cdCell.condition);
                                                    }
                                                }
                                                if (this.ch != ')') {
                                                    this.ret = 3;
                                                    return;
                                                }
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(")");
                                                }
                                                this.getChar();
                                                this.ret = this.getStatData(syData.cdCell, syData.value);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(syData.cdCell);
                                                }
                                                syData.assign(syData.value);
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan("SUM");
                                                }
                                                this.ret = 0;
                                                return;
                                            }
                                        }
                                    }
                                }
                                else {
                                    final int saveExpI = this.expi;
                                    final char saveCh = this.ch;
                                    if (this.ch == ' ') {
                                        this.skipSpaces();
                                    }
                                    if (this.ch == '(') {
                                        final FunctionListener funcListener = this.getFunctionListener(this.tmpStr);
                                        if (funcListener == null) {
                                            this.ret = 201;
                                            return;
                                        }
                                        final Function func = funcListener.funcByName(this.tmpStr);
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(String.valueOf(func.title) + '(');
                                        }
                                        final CommonDatas paramValues = new CommonDatas();
                                        for (int i = 0; i < func.params.size(); ++i) {
                                            paramValues.add(new CommonData());
                                        }
                                        for (int i = 0; i < func.params.size(); ++i) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != ',' && this.ch != ')') {
                                                this.multorFuncParam(func.params.getParam(i), leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                paramValues.getItem(i).assign(leftOpData);
                                            }
                                            else {
                                                if (!func.params.getParam(i).canOmit) {
                                                    this.ret = 202;
                                                    return;
                                                }
                                                paramValues.getItem(i).type = func.params.getParam(i).type;
                                                paramValues.getItem(i).isNull = true;
                                                paramValues.getItem(i).initNullValue();
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(paramValues.getItem(i));
                                                }
                                            }
                                            if (i < func.params.size() - 1) {
                                                if (this.ch == ' ') {
                                                    this.skipSpaces();
                                                }
                                                if (this.ch == ',') {
                                                    this.getChar();
                                                    if (this.hasExpDescription) {
                                                        this.addStringCaption(',');
                                                    }
                                                }
                                            }
                                        }
                                        if (func.isInfiniteParam && func.params.size() > 0) {
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            while (this.ch == ',') {
                                                this.getChar();
                                                if (this.hasExpDescription) {
                                                    this.addStringCaption(',');
                                                }
                                                this.multorFuncParam(func.params.getParam(func.params.size() - 1), leftOpData);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                paramValues.add(new CommonData());
                                                paramValues.getItem(paramValues.size() - 1).assign(leftOpData);
                                                if (this.ch != ' ') {
                                                    continue;
                                                }
                                                this.skipSpaces();
                                            }
                                        }
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch != ')') {
                                            this.ret = 3;
                                            return;
                                        }
                                        this.getChar();
                                        if (this.hasExpDescription) {
                                            this.addStringCaption(')');
                                        }
                                        if (func.isInfiniteParam && this.hasMakeNiporlan) {
                                            this.tempSyData.type = 1;
                                            this.tempSyData.isNull = false;
                                            this.tempSyData.cdInt = paramValues.size();
                                            this.writeNiporlan(this.tempSyData);
                                        }
                                        if (this.hasMakeNiporlan) {
                                            this.writeNiporlan(func.name);
                                        }
                                        syData.type = func.resultType;
                                        syData.isNull = false;
                                        syData.initValue();
                                        this.ret = funcListener.callFunction(func.name, paramValues, syData, this.isCheckMode || !this.canAssignValue);
                                        return;
                                    }
                                    else {
                                        this.expi = saveExpI;
                                        this.ch = saveCh;
                                        if (this.ch == ' ') {
                                            this.skipSpaces();
                                        }
                                        if (this.ch == '[' && this.hasCellSyntax) {
                                            this.expi = saveExpI - this.tmpStr.length();
                                            this.ch = this.tmpStr.charAt(0);
                                            this.parseCellExp(syData);
                                            if (this.ret != 0) {
                                                return;
                                            }
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            if (this.ch != '&') {
                                                if (this.hasExpDescription) {
                                                    this.addCellCaption(syData.cdCell);
                                                }
                                                if (this.hasExpVision) {
                                                    this.addCellVision(syData.cdCell);
                                                }
                                                if (this.hasMakeNiporlan) {
                                                    this.writeNiporlan(syData.cdCell);
                                                }
                                                this.ret = 0;
                                                return;
                                            }
                                            this.getChar();
                                            if (this.ch == ' ') {
                                                this.skipSpaces();
                                            }
                                            this.parseCellExp(leftOpData);
                                            if (this.ret != 0) {
                                                this.ret = 153;
                                                return;
                                            }
                                            if (syData.cdCell.isZB || leftOpData.cdCell.isZB) {
                                                this.ret = 153;
                                                return;
                                            }
                                            if (this.hasExpDescription) {
                                                this.addStringCaption(String.valueOf(syData.cdCell.toString()) + '&' + leftOpData.cdCell.toString());
                                            }
                                            if (this.hasMakeNiporlan) {
                                                this.tempSyData.assign(syData);
                                                for (int r = syData.cdCell.row; r <= leftOpData.cdCell.row; ++r) {
                                                    for (int c = syData.cdCell.col; c <= leftOpData.cdCell.col; ++c) {
                                                        this.tempSyData.cdCell.row = r;
                                                        this.tempSyData.cdCell.col = c;
                                                        this.ret = this.getCellData(this.tempSyData.cdCell, this.tempSyData.value);
                                                        if (this.ret != 0) {
                                                            return;
                                                        }
                                                        this.writeNiporlan(this.tempSyData.cdCell);
                                                        if (r != syData.cdCell.row || c != syData.cdCell.col) {
                                                            this.writeNiporlan(1);
                                                        }
                                                    }
                                                }
                                            }
                                            this.checkOpType(syData, leftOpData, 6);
                                            if (this.ret != 0) {
                                                this.ret = 153;
                                                return;
                                            }
                                            this.ret = 0;
                                            return;
                                        }
                                    }
                                }
                            }
                            this.ret = 102;
                            break Label_17163;
                        }
                        this.parseCellExp(syData);
                        if (this.ret == 0) {
                            if (this.ch == ' ') {
                                this.skipSpaces();
                            }
                            if (this.ch == '&') {
                                this.getChar();
                                if (this.ch == ' ') {
                                    this.skipSpaces();
                                }
                                this.parseCellExp(leftOpData);
                                if (this.ret != 0) {
                                    this.ret = 153;
                                }
                                else if (syData.cdCell.isZB || leftOpData.cdCell.isZB) {
                                    this.ret = 153;
                                }
                                else {
                                    if (this.hasExpDescription) {
                                        this.addStringCaption(String.valueOf(syData.cdCell.toString()) + '&' + leftOpData.cdCell.toString());
                                    }
                                    if (this.hasExpVision) {
                                        this.addStringVision(String.valueOf(syData.cdCell.toString()) + '&' + leftOpData.cdCell.toString());
                                    }
                                    if (this.hasMakeNiporlan) {
                                        this.tempSyData.assign(syData);
                                        for (int r2 = syData.cdCell.row; r2 <= leftOpData.cdCell.row; ++r2) {
                                            for (int c2 = syData.cdCell.col; c2 <= leftOpData.cdCell.col; ++c2) {
                                                this.tempSyData.cdCell.row = r2;
                                                this.tempSyData.cdCell.col = c2;
                                                this.ret = this.getCellData(this.tempSyData.cdCell, this.tempSyData.value);
                                                if (this.ret != 0) {
                                                    return;
                                                }
                                                this.writeNiporlan(this.tempSyData.cdCell);
                                                if (r2 != syData.cdCell.row || c2 != syData.cdCell.col) {
                                                    this.writeNiporlan(1);
                                                }
                                            }
                                        }
                                    }
                                    this.checkOpType(syData, leftOpData, 6);
                                    if (this.ret != 0) {
                                        this.ret = 153;
                                    }
                                    else {
                                        this.ret = 0;
                                    }
                                }
                            }
                            else {
                                if (this.hasExpDescription) {
                                    this.addCellCaption(syData.cdCell);
                                }
                                if (this.hasExpVision) {
                                    this.addCellVision(syData.cdCell);
                                }
                                if (this.hasMakeNiporlan) {
                                    this.writeNiporlan(syData.cdCell);
                                }
                                this.ret = 0;
                            }
                        }
                    }
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void product(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_0230: {
            try {
                char saveCh = '\0';
                this.multor(leftOpData);
                if (this.ret == 0) {
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    while (this.ch == '*' || this.ch == '/') {
                        saveCh = this.ch;
                        if (this.hasExpDescription) {
                            this.addStringCaption(this.ch);
                        }
                        this.getChar();
                        this.multor(rightOpData);
                        if (this.ret != 0) {
                            return;
                        }
                        if (saveCh == '*') {
                            this.checkOpType(leftOpData, rightOpData, 4);
                            if (this.hasMakeNiporlan && !this.inParseCell) {
                                this.writeNiporlan(4);
                            }
                        }
                        else if (saveCh == '/') {
                            this.checkOpType(leftOpData, rightOpData, 5);
                            if (this.hasMakeNiporlan && !this.inParseCell) {
                                this.writeNiporlan(5);
                            }
                        }
                        if (this.ret != 0) {
                            return;
                        }
                        if (this.ch != ' ') {
                            continue;
                        }
                        this.skipSpaces();
                    }
                    syData.assign(leftOpData);
                    this.ret = 0;
                    break Label_0230;
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void subItem(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_0373: {
            try {
                int prefixsy = 0;
                char saveCh = '\0';
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch == '+') {
                    ++prefixsy;
                    rightOpData.type = 1;
                    rightOpData.cdInt = 1;
                    this.getChar();
                    if (this.hasExpDescription) {
                        this.addStringCaption("+");
                    }
                }
                if (this.ch == '-') {
                    prefixsy += 2;
                    rightOpData.type = 1;
                    rightOpData.cdInt = -1;
                    this.getChar();
                    if (this.hasExpDescription) {
                        this.addStringCaption("-");
                    }
                }
                if (prefixsy == 3) {
                    this.ret = 151;
                }
                else {
                    this.product(leftOpData);
                    if (this.ret == 0) {
                        if (prefixsy == 2) {
                            this.checkOpType(leftOpData, rightOpData, 4);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan && !this.inParseCell) {
                                this.writeNiporlan(3);
                            }
                        }
                        if (this.ch == ' ') {
                            this.skipSpaces();
                        }
                        while (this.ch == '+' || this.ch == '-') {
                            saveCh = this.ch;
                            if (this.hasExpDescription) {
                                this.addStringCaption(saveCh);
                            }
                            this.getChar();
                            this.product(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            if (saveCh == '+') {
                                this.checkOpType(leftOpData, rightOpData, 1);
                                if (this.hasMakeNiporlan && !this.inParseCell) {
                                    this.writeNiporlan(1);
                                }
                            }
                            else {
                                this.checkOpType(leftOpData, rightOpData, 2);
                                if (this.hasMakeNiporlan && !this.inParseCell) {
                                    this.writeNiporlan(2);
                                }
                            }
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.ch != ' ') {
                                continue;
                            }
                            this.skipSpaces();
                        }
                        syData.assign(leftOpData);
                        this.ret = 0;
                        break Label_0373;
                    }
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void mainItem(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_0852: {
            try {
                this.subItem(leftOpData);
                if (this.ret == 0) {
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    if (isCompareChar(this.ch)) {
                        this.tmpStrBuffer.setLength(0);
                        if (isCompareChar(this.ch)) {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                            while (isCompareChar(this.ch)) {
                                this.tmpStrBuffer.append(this.ch);
                                this.getChar();
                            }
                        }
                        else {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                            while (isSignChar(this.ch)) {
                                this.tmpStrBuffer.append(this.ch);
                                this.getChar();
                            }
                        }
                        this.tmpStr = this.tmpStrBuffer.toString();
                        if (this.tmpStr.equals("=") || this.tmpStr.equals(SyntaxParser.OP_EQU_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption("=");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 7);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(7);
                            }
                        }
                        else if (this.tmpStr.equals("==") || this.tmpStr.equals(SyntaxParser.OP_WEQU_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption("==");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 8);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(8);
                            }
                        }
                        else if (this.tmpStr.toString().equals("<>") || this.tmpStr.toString().equals(SyntaxParser.OP_NEQ_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption("<>");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 9);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(9);
                            }
                        }
                        else if (this.tmpStr.equals(">") || this.tmpStr.equals(SyntaxParser.OP_GLT_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption(">");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 10);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(10);
                            }
                        }
                        else if (this.tmpStr.equals("<") || this.tmpStr.equals(SyntaxParser.OP_BLT_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption("<");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 11);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(11);
                            }
                        }
                        else if (this.tmpStr.equals(">=") || this.tmpStr.equals(SyntaxParser.OP_GLE_CHINA_NAME)) {
                            if (this.hasExpDescription) {
                                this.addStringCaption(">=");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 12);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(12);
                            }
                        }
                        else {
                            if (!this.tmpStr.equals("<=") && !this.tmpStr.equals(SyntaxParser.OP_BLE_CHINA_NAME)) {
                                this.ret = 102;
                                this.expi = this.expi - this.tmpStr.length() - 1;
                                this.getChar();
                                return;
                            }
                            if (this.hasExpDescription) {
                                this.addStringCaption("<=");
                            }
                            this.subItem(rightOpData);
                            if (this.ret != 0) {
                                return;
                            }
                            this.checkOpType(leftOpData, rightOpData, 13);
                            if (this.ret != 0) {
                                return;
                            }
                            if (this.hasMakeNiporlan) {
                                this.writeNiporlan(13);
                            }
                        }
                    }
                    syData.assign(leftOpData);
                    this.ret = 0;
                    break Label_0852;
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void simpleExp(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_0299: {
            try {
                this.mainItem(leftOpData);
                if (this.ret == 0) {
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    while (isFirstSignChar(this.ch)) {
                        this.tmpStrBuffer.setLength(0);
                        this.tmpStrBuffer.append(this.ch);
                        this.getChar();
                        while (isSignChar(this.ch)) {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                        }
                        this.tmpStr = this.tmpStrBuffer.toString();
                        if (!this.tmpStr.equals("AND") && !this.tmpStr.equals(SyntaxParser.OP_AND_CHINA_NAME)) {
                            this.expi = this.expi - this.tmpStr.length() - 1;
                            this.getChar();
                            break;
                        }
                        if (this.hasExpDescription) {
                            this.addStringCaption(" " + SyntaxParser.OP_AND_CHINA_NAME + " ");
                        }
                        this.mainItem(rightOpData);
                        if (this.ret != 0) {
                            return;
                        }
                        this.checkOpType(leftOpData, rightOpData, 14);
                        if (this.ret != 0) {
                            return;
                        }
                        if (this.hasMakeNiporlan) {
                            this.writeNiporlan(14);
                        }
                        if (this.ch != ' ') {
                            continue;
                        }
                        this.skipSpaces();
                    }
                    syData.assign(leftOpData);
                    this.ret = 0;
                    break Label_0299;
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void wholeExp(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        final SyntaxData rightOpData = this.newSyntaxData();
        Label_0299: {
            try {
                this.simpleExp(leftOpData);
                if (this.ret == 0) {
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    while (isFirstSignChar(this.ch)) {
                        this.tmpStrBuffer.setLength(0);
                        this.tmpStrBuffer.append(this.ch);
                        this.getChar();
                        while (isSignChar(this.ch)) {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                        }
                        this.tmpStr = this.tmpStrBuffer.toString();
                        if (!this.tmpStr.equals("OR") && !this.tmpStr.equals(SyntaxParser.OP_OR_CHINA_NAME)) {
                            this.expi = this.expi - this.tmpStr.length() - 1;
                            this.getChar();
                            break;
                        }
                        if (this.hasExpDescription) {
                            this.addStringCaption(" " + SyntaxParser.OP_OR_CHINA_NAME + " ");
                        }
                        this.simpleExp(rightOpData);
                        if (this.ret != 0) {
                            return;
                        }
                        this.checkOpType(leftOpData, rightOpData, 15);
                        if (this.ret != 0) {
                            return;
                        }
                        if (this.hasMakeNiporlan) {
                            this.writeNiporlan(15);
                        }
                        if (this.ch != ' ') {
                            continue;
                        }
                        this.skipSpaces();
                    }
                    syData.assign(leftOpData);
                    this.ret = 0;
                    break Label_0299;
                }
                return;
            }
            finally {
                this.disposeSyntaxData(leftOpData);
                this.disposeSyntaxData(rightOpData);
            }
        }
        this.disposeSyntaxData(leftOpData);
        this.disposeSyntaxData(rightOpData);
    }
    
    private void parseCellExp(final SyntaxData syData) {
        final SyntaxData leftOpData = this.newSyntaxData();
        int tempTerm = 0;
        this.inParseCell = true;
        try {
            syData.type = 9;
            syData.cdCell.init();
            if (this.ch == ' ') {
                this.skipSpaces();
            }
            Label_0184: {
                if (this.ch == '[') {
                    this.tmpStr = "";
                    break Label_0184;
                }
                if (!isFirstSignChar(this.ch)) {
                    this.ret = 156;
                }
                else {
                    this.tmpStrBuffer.setLength(0);
                    this.tmpStrBuffer.append(this.ch);
                    this.getChar();
                    while (isSignChar(this.ch)) {
                        this.tmpStrBuffer.append(this.ch);
                        this.getChar();
                        if (this.tmpStrBuffer.length() > 50) {
                            this.ret = 157;
                            return;
                        }
                    }
                    this.tmpStr = this.tmpStrBuffer.toString();
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    break Label_0184;
                }
                return;
            }
            if (this.ch == '[') {
                syData.cdCell.tableName = this.tmpStr;
                this.getChar();
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (isFirstSignChar(this.ch)) {
                    this.tmpStrBuffer.setLength(0);
                    this.tmpStrBuffer.append(this.ch);
                    this.getChar();
                    while (isSignChar(this.ch)) {
                        this.tmpStrBuffer.append(this.ch);
                        this.getChar();
                        if (this.tmpStrBuffer.length() > 50) {
                            this.ret = 157;
                            return;
                        }
                    }
                    this.tmpStr = this.tmpStrBuffer.toString();
                    syData.cdCell.isZB = true;
                    syData.cdCell.zbName = this.tmpStr;
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    if (this.ch == '$') {
                        syData.cdCell.fetchMean = true;
                        this.getChar();
                    }
                }
                else {
                    this.subItem(leftOpData);
                    if (this.ret != 0 || leftOpData.type != 1) {
                        this.ret = 15;
                        return;
                    }
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    if (this.ch != ',') {
                        this.ret = 14;
                        return;
                    }
                    syData.cdCell.isZB = false;
                    syData.cdCell.row = leftOpData.cdInt;
                    this.getChar();
                    this.subItem(leftOpData);
                    if (this.ret != 0 || leftOpData.type != 1) {
                        this.ret = 15;
                        return;
                    }
                    syData.cdCell.col = leftOpData.cdInt;
                }
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                while (this.ch == ',') {
                    this.getChar();
                    if (this.ch == ' ') {
                        this.skipSpaces();
                    }
                    if (this.ch == '\'') {
                        if (syData.cdCell.hasCond) {
                            this.ret = 158;
                            return;
                        }
                        syData.cdCell.hasCond = true;
                        this.getChar();
                        this.tmpStrBuffer.setLength(0);
                        while (this.ch != '\'' && this.ch != '\0') {
                            this.tmpStrBuffer.append(this.ch);
                            this.getChar();
                        }
                        if (this.ch != '\'') {
                            this.ret = 159;
                            return;
                        }
                        syData.cdCell.condition = this.tmpStrBuffer.toString();
                        this.getChar();
                        if (this.ch != ' ') {
                            continue;
                        }
                        this.skipSpaces();
                    }
                    else {
                        if (isDigitChar(this.ch) || this.ch == '-' || this.ch == '+') {
                            tempTerm = this.getInteger();
                            if (syData.cdCell.hasCond) {
                                if ("".equals(DataCell.statIntToSign(tempTerm + 1))) {
                                    this.ret = 156;
                                    return;
                                }
                                syData.cdCell.statMode = tempTerm;
                                if (this.ch == ' ') {
                                    this.skipSpaces();
                                    continue;
                                }
                                continue;
                            }
                            else {
                                boolean isNext = false;
                                if (this.ch == '+') {
                                    isNext = true;
                                }
                                if (isNext) {
                                    tempTerm += 10000;
                                }
                                if (this.ret != 0) {
                                    this.ret = 156;
                                    return;
                                }
                                if (this.ch == ' ') {
                                    this.skipSpaces();
                                }
                                this.tmpStr = "";
                                if (isLetterChar(this.ch) || isChinaChar(this.ch)) {
                                    this.tmpStrBuffer.setLength(0);
                                    this.tmpStrBuffer.append(this.ch);
                                    this.getChar();
                                    while (isLetterChar(this.ch) || isChinaChar(this.ch)) {
                                        this.tmpStrBuffer.append(this.ch);
                                        this.getChar();
                                    }
                                    this.tmpStr = this.tmpStrBuffer.toString();
                                }
                            }
                        }
                        else {
                            this.tmpStrBuffer.setLength(0);
                            while (isLetterChar(this.ch) || isChinaChar(this.ch)) {
                                this.tmpStrBuffer.append(this.ch);
                                this.getChar();
                            }
                            this.tmpStr = this.tmpStrBuffer.toString();
                            if (this.ch == ' ') {
                                this.skipSpaces();
                            }
                            if (DataCell.statSignToInt(this.tmpStr) >= 0) {
                                if (syData.cdCell.statMode != 0) {
                                    this.ret = 162;
                                    return;
                                }
                                syData.cdCell.statMode = DataCell.statSignToInt(this.tmpStr);
                                continue;
                            }
                            else {
                                this.tmpInt = DataCell.kindSignToInt(this.tmpStr);
                                if (this.tmpInt != 0) {
                                    if (isDigitChar(this.ch) || this.ch == '-' || this.ch == '+') {
                                        boolean isNext = false;
                                        if (this.ch == '+') {
                                            isNext = true;
                                        }
                                        tempTerm = this.getInteger();
                                        if (isNext) {
                                            tempTerm += 10000;
                                        }
                                        if (this.ret != 0) {
                                            this.ret = 156;
                                            return;
                                        }
                                    }
                                    else {
                                        tempTerm = 0;
                                    }
                                }
                                else {
                                    while (isLetterChar(this.ch) || isDigitChar(this.ch)) {
                                        this.tmpStr = String.valueOf(this.tmpStr) + this.ch;
                                        this.getChar();
                                    }
                                }
                            }
                        }
                        if (this.tmpStr.equals("")) {
                            if (syData.cdCell.termKind != 0 || syData.cdCell.term != 0) {
                                this.ret = 160;
                                return;
                            }
                            syData.cdCell.termKind = 0;
                            syData.cdCell.term = tempTerm;
                        }
                        else {
                            this.tmpInt = DataCell.kindSignToInt(this.tmpStr);
                            if (this.tmpInt == 0) {
                                if (this.ch == ' ') {
                                    this.skipSpaces();
                                }
                                if (this.ch != '=') {
                                    this.ret = 156;
                                    return;
                                }
                                this.getChar();
                                if (this.ch == ' ') {
                                    this.skipSpaces();
                                }
                                this.tmpStrBuffer.setLength(0);
                                if (this.ch == '\"') {
                                    this.getChar();
                                    while (this.ch != '\"' && this.ch != '\0') {
                                        this.tmpStrBuffer.append(this.ch);
                                        this.getChar();
                                    }
                                    if (this.ch != '\"') {
                                        this.ret = 156;
                                        return;
                                    }
                                    this.getChar();
                                }
                                else if (this.ch == '\'') {
                                    this.getChar();
                                    while (this.ch != '\'' && this.ch != '\0') {
                                        this.tmpStrBuffer.append(this.ch);
                                        this.getChar();
                                    }
                                    if (this.ch != '\'') {
                                        this.ret = 156;
                                        return;
                                    }
                                    this.getChar();
                                }
                                else {
                                    while (this.ch != ' ' && this.ch != ']' && this.ch != ',' && this.ch != '\0') {
                                        this.tmpStrBuffer.append(this.ch);
                                        this.getChar();
                                    }
                                }
                                if (syData.cdCell.dimens == null) {
                                    syData.cdCell.dimens = new HashMap(2);
                                }
                                if (syData.cdCell.dimens.get(this.tmpStr) != null) {
                                    this.ret = 163;
                                    return;
                                }
                                syData.cdCell.dimens.put(this.tmpStr, this.tmpStrBuffer.toString());
                            }
                            else if (this.tmpInt == 1) {
                                if (tempTerm == 0) {
                                    if (syData.cdCell.termKind != 0) {
                                        this.ret = 160;
                                        return;
                                    }
                                    syData.cdCell.termKind = 1;
                                    syData.cdCell.term = 0;
                                }
                                else {
                                    if (syData.cdCell.year != 0) {
                                        this.ret = 160;
                                        return;
                                    }
                                    syData.cdCell.year = tempTerm;
                                }
                            }
                            else {
                                if (syData.cdCell.termKind != 0 || syData.cdCell.term != 0) {
                                    this.ret = 160;
                                    return;
                                }
                                syData.cdCell.termKind = this.tmpInt;
                                syData.cdCell.term = tempTerm;
                            }
                        }
                        if (this.ch != ' ') {
                            continue;
                        }
                        this.skipSpaces();
                    }
                }
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch != ']') {
                    this.ret = 5;
                    return;
                }
                this.getChar();
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch == '$') {
                    syData.cdCell.fetchMean = true;
                    this.getChar();
                }
            }
            else {
                syData.cdCell.tableName = "";
                syData.cdCell.isZB = true;
                syData.cdCell.zbName = this.tmpStr;
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch == '$') {
                    syData.cdCell.fetchMean = true;
                    this.getChar();
                }
            }
            this.ret = this.getCellData(syData.cdCell, syData.value);
        }
        finally {
            this.inParseCell = false;
            this.disposeSyntaxData(leftOpData);
        }
        this.inParseCell = false;
        this.disposeSyntaxData(leftOpData);
    }
    
    private void expSubCheck(final SyntaxData syData, final SyntaxData leftOpData) {
        this.expi = 0;
        this.getChar();
        if (this.estimateType == 1) {
            if (this.ch == ' ') {
                this.skipSpaces();
            }
            this.tmpStrBuffer.setLength(0);
            while (isLetterChar(this.ch)) {
                this.tmpStrBuffer.append(this.ch);
                this.getChar();
            }
            this.tmpStr = this.tmpStrBuffer.toString();
            if (this.tmpStr.equalsIgnoreCase("CALL")) {
                this.wholeExp(syData);
                if (this.ret != 0) {
                    return;
                }
                if (this.ch != '\0') {
                    this.ret = 101;
                }
            }
            else {
                this.expi = 0;
                this.getChar();
                this.parseCellExp(syData);
                if (this.ret != 0) {
                    return;
                }
                if (this.ch == ' ') {
                    this.skipSpaces();
                }
                if (this.ch != '=') {
                    this.ret = 13;
                    return;
                }
                if (this.hasMakeNiporlan) {
                    this.writeNiporlan(syData.cdCell);
                }
                this.getChar();
                this.wholeExp(leftOpData);
                if (this.ret != 0) {
                    return;
                }
                this.checkOpType(syData, leftOpData, 17);
                if (this.ret != 0) {
                    return;
                }
                if (this.hasMakeNiporlan) {
                    this.writeNiporlan(17);
                }
                if (this.ch != '\0') {
                    this.ret = 101;
                }
            }
        }
        else {
            this.wholeExp(syData);
            if (this.ret != 0) {
                return;
            }
            if (this.ch != '\0') {
                this.ret = 101;
                return;
            }
            if (this.estimateType == 0) {
                if (syData.type != 5) {
                    this.ret = 19;
                    return;
                }
                if (this.hasExpVisionBanlance) {
                    final List splitList = new ArrayList();
                    splitExp(this.expStr, splitList);
                    for (int i = 0; i < splitList.size(); ++i) {
                        final String exptemp = splitList.get(i);
                        this.tmpStr = makeLeftExp(exptemp);
                        if (!this.tmpStr.equals("")) {
                            this.addStringVision(String.valueOf(this.tmpStr) + "//左");
                        }
                        this.tmpStr = makeRightExp(exptemp);
                        if (!this.tmpStr.equals("")) {
                            this.addStringVision(String.valueOf(this.tmpStr) + "//右");
                        }
                        this.tmpStr = makeBanlanceExp(exptemp);
                        if (!this.tmpStr.equals("")) {
                            this.addStringVision(String.valueOf(this.tmpStr) + "//差额");
                        }
                    }
                }
            }
            else {
                switch (syData.type) {
                    case 9: {
                        syData.assign(syData.value);
                        break;
                    }
                }
            }
        }
    }
    
    private void expCheck(final SyntaxData syData) {
        if (this.hasMakeNiporlan) {
            this.niporlan = new Niporlan();
        }
        if (this.hasExpDescription) {
            this.expDescription = "";
        }
        if (this.hasExpVision && (this.defineExpVision == null || this.defineExpVision.equals(""))) {
            this.expVision = "";
        }
        this.ifExpStr = "";
        this.thenExpStr = "";
        this.elseExpStr = "";
        this.wholeExpStr = this.expStr;
        final SyntaxData leftOpData = this.newSyntaxData();
        this.canAssignValue = true;
        this.niporlanItemType = 1;
        this.inEffectCheck = true;
        this.ret = 0;
        Label_1128: {
            try {
                if (syntaxPos("IF", this.expStr) == 0) {
                    this.thenPos = syntaxPos("THEN", this.expStr);
                    if (this.thenPos < 0) {
                        this.ret = 23;
                    }
                    else {
                        this.ifExpStr = this.expStr.substring(2, this.thenPos);
                        if (this.ifExpStr.equals("")) {
                            this.ret = 103;
                        }
                        else {
                            this.elsePos = syntaxPos("ELSE", this.expStr);
                            if (this.elsePos < 0) {
                                this.thenExpStr = this.expStr.substring(this.thenPos + 4);
                                if (this.thenExpStr.equals("")) {
                                    this.ret = 104;
                                    return;
                                }
                            }
                            else {
                                this.thenExpStr = this.expStr.substring(this.thenPos + 4, this.elsePos);
                                this.elseExpStr = this.expStr.substring(this.elsePos + 4);
                                if (this.thenExpStr.equals("")) {
                                    this.ret = 104;
                                    return;
                                }
                                if (this.elseExpStr.equals("")) {
                                    this.ret = 105;
                                    return;
                                }
                            }
                            this.ifExpStr = this.ifExpStr.trim();
                            this.thenExpStr = this.thenExpStr.trim();
                            this.elseExpStr = this.elseExpStr.trim();
                            this.expStr = this.ifExpStr;
                            this.niporlanItemType = 0;
                            if (this.hasExpDescription) {
                                this.addStringCaption(String.valueOf(SyntaxParser.OP_IF_CHINA_NAME) + " ");
                            }
                            this.expi = 0;
                            this.getChar();
                            this.wholeExp(syData);
                            Label_0529: {
                                try {
                                    if (this.ret == 0) {
                                        if (this.ch != '\0') {
                                            this.ret = 101;
                                        }
                                        else {
                                            if (syData.type == 5) {
                                                break Label_0529;
                                            }
                                            this.ret = 19;
                                        }
                                    }
                                    return;
                                }
                                finally {
                                    if (this.ret != 0) {
                                        this.expi += "IF".length();
                                    }
                                }
                            }
                            if (this.ret != 0) {
                                this.expi += "IF".length();
                            }
                            this.ifThenTrue = syData.cdBool;
                            this.expStr = this.thenExpStr;
                            this.niporlanItemType = 1;
                            if (this.hasExpDescription) {
                                this.addStringCaption(" " + SyntaxParser.OP_THEN_CHINA_NAME + " ");
                            }
                            this.expi = 0;
                            this.getChar();
                            this.canAssignValue = (this.isIgnoreControl || this.ifThenTrue);
                            this.expSubCheck(syData, leftOpData);
                            Label_0712: {
                                try {
                                    if (this.ret == 0) {
                                        if (this.ch == '\0') {
                                            break Label_0712;
                                        }
                                        this.ret = 101;
                                    }
                                    return;
                                }
                                finally {
                                    if (this.ret != 0) {
                                        this.expi += "THEN".length();
                                    }
                                }
                            }
                            if (this.ret != 0) {
                                this.expi += "THEN".length();
                            }
                            if (this.estimateType == 0 && this.ifThenTrue) {
                                this.judgeResult = syData.cdBool;
                            }
                            if (this.estimateType == 2 && this.ifThenTrue) {
                                this.attractResult.assign(syData);
                            }
                            if (this.elsePos >= 0) {
                                this.expStr = this.elseExpStr;
                                this.niporlanItemType = 2;
                                if (this.hasExpDescription) {
                                    this.addStringCaption(" " + SyntaxParser.OP_ELSE_CHINA_NAME + " ");
                                }
                                this.expi = 0;
                                this.getChar();
                                this.canAssignValue = (this.isIgnoreControl || !this.ifThenTrue);
                                this.expSubCheck(syData, leftOpData);
                                Label_0947: {
                                    try {
                                        if (this.ret == 0) {
                                            if (this.ch == '\0') {
                                                break Label_0947;
                                            }
                                            this.ret = 101;
                                        }
                                        return;
                                    }
                                    finally {
                                        if (this.ret != 0) {
                                            this.expi += "ELSE".length();
                                        }
                                    }
                                }
                                if (this.ret != 0) {
                                    this.expi += "ELSE".length();
                                }
                                if (this.estimateType == 0 && !this.ifThenTrue) {
                                    this.judgeResult = syData.cdBool;
                                }
                                if (this.estimateType == 2 && !this.ifThenTrue) {
                                    this.attractResult.assign(syData);
                                }
                            }
                            else {
                                if (this.estimateType == 0 && !this.ifThenTrue) {
                                    this.judgeResult = true;
                                }
                                if (this.estimateType == 2 && !this.ifThenTrue) {
                                    this.attractResult.type = 0;
                                }
                            }
                            if (this.estimateType == 0) {
                                syData.type = 5;
                                syData.cdBool = this.judgeResult;
                            }
                            if (this.estimateType == 2) {
                                syData.assign(this.attractResult);
                            }
                            break Label_1128;
                        }
                    }
                }
                else {
                    this.expSubCheck(syData, leftOpData);
                    if (this.ret == 0) {
                        break Label_1128;
                    }
                }
                return;
            }
            finally {
                this.inEffectCheck = false;
                this.disposeSyntaxData(leftOpData);
            }
        }
        this.inEffectCheck = false;
        this.disposeSyntaxData(leftOpData);
    }
    
    public static void main1(final String[] args) {
        System.out.println("Start Syntax Testing!");
        final SyntaxParser parser = new SyntaxParser();
        final CommonData resultData = new CommonData();
        parser.hasExpDescription = true;
        parser.hasExpVision = true;
        parser.hasMakeNiporlan = true;
        System.out.println("start:" + System.currentTimeMillis() / 1000L);
        int ret = 0;
        ret = parser.attract(" floatcopy        ( '11111' , '222 \"\" 222', '4444' , '5555' ,1, true)", resultData);
        System.out.println("end:" + System.currentTimeMillis() / 1000L);
        System.out.println("ret:" + ret);
        if (ret != 0) {
            System.out.println("error message:" + parser.getErrorMessage());
        }
        System.out.println("expstr:" + parser.getExpStr());
        System.out.println("IF expstr:" + parser.getExpStrIfPart());
        System.out.println("THEN expstr:" + parser.getExpStrThenPart());
        System.out.println("ELSE expstr:" + parser.getExpStrElsePart());
        System.out.println("expi:" + parser.expi);
        System.out.println("expDescription:" + parser.getExpDescription());
        System.out.println("expVision:" + parser.getExpVision());
        System.out.println("result.type:" + resultData.type);
        System.out.println("result value:" + resultData.toString());
        System.out.println("<niporlan start!>");
        System.out.print("niporlan if's count:");
        System.out.println((parser.niporlan.ifItems == null) ? 0 : parser.niporlan.ifItems.size());
        System.out.println("niporlan then's count:" + parser.niporlan.thenItems.size());
        System.out.print("niporlan else's count:");
        System.out.println((parser.niporlan.elseItems == null) ? 0 : parser.niporlan.elseItems.size());
        resultData.type = 0;
        resultData.cdString = "";
        parser.attract(parser.getNiporlan(), resultData);
        System.out.println("result.type:" + resultData.type);
        System.out.println("result value:" + resultData.toString());
        final CommonTransJavaScriptListener listenter = new CommonTransJavaScriptListener();
        System.out.print("JavaScript:");
        System.out.println(parser.transNiporlanToJavaScript(parser.niporlan, false, listenter));
    }
}
